# -*- coding: utf-8 -*-
#
#  License:  AGPLv3
#  contact:  http://www.privacyidea.org
#
#  2014-10-04 Cornelius Kölbel <cornelius@privacyidea.org>
#
# This code is free software; you can redistribute it and/or
# modify it under the terms of the GNU AFFERO GENERAL PUBLIC LICENSE
# License as published by the Free Software Foundation; either
# version 3 of the License, or any later version.
#
# This code is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU AFFERO GENERAL PUBLIC LICENSE for more details.
#
# You should have received a copy of the GNU Affero General Public
# License along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
'''
Description:  functional tests of ldap resolver
'''
import logging

from mockldap import MockLdap
from privacyidea.tests import TestController, url
# from privacyidea.lib.resolvers import LDAPIdResolver
from privacyidea.lib.resolvers.UserIdResolver import getResolverClass

log = logging.getLogger(__name__)


class TestLDAPResolver(TestController):
    top = ('o=test', {'o': 'test'})
    example = ('ou=example,o=test', {'ou': 'example'})
    other = ('ou=other,o=test', {'ou': 'other'})
    
    alice = ('cn=alice,ou=example,o=test', {'dn': 'cn=alice,ou=example,o=test',
                                            'cn': 'alice',
                                            'userPassword': ['alicepw']})
    bob = ('cn=bob,ou=other,o=test', {'dn': 'cn=bob,ou=other,o=test',
                                      'cn': 'bob',
                                      'userPassword': ['bobpw']})
    admin = ('cn=admin,o=test', {'dn': 'cn=admin,o=test',
                                 'cn': 'admin',
                                 'userPassword': ['secret']})

    # This is the content of our mock LDAP directory. It takes the form
    # {dn: {attr: [value, ...], ...}, ...}.
    directory = dict([top, example, other,
                      alice,
                      bob,
                      admin])
    mockldap = MockLdap(directory)

    @classmethod
    def tearDownClass(cls):
        del cls.mockldap

    def setUp(self):
        # Patch ldap.initialize
        self.mockldap.start()
        self.ldapobj = self.mockldap['ldap://localhost/']

    def tearDown(self):
        # Stop patching ldap.initialize and reset state.
        self.mockldap.stop()
        del self.ldapobj

    def test_01_standalone_DN(self):
        '''
        Testing standalone functionality with UIDTYPE DN
        '''
        y = getResolverClass("LDAPIdResolver", "IdResolver")()
    
        y.loadConfig({'privacyidea.ldapresolver.LDAPURI': 'ldap://localhost',
                      'privacyidea.ldapresolver.LDAPBASE': 'o=test',
                      'privacyidea.ldapresolver.BINDDN': 'cn=admin,o=test',
                      'privacyidea.ldapresolver.BINDPW': 'secret',
                      'privacyidea.ldapresolver.LOGINNAMEATTRIBUTE': 'cn',
                      'privacyidea.ldapresolver.LDAPSEARCHFILTER': '(cn=*)',
                      'privacyidea.ldapresolver.LDAPFILTER': '(&(cn=%s))',
                      'privacyidea.ldapresolver.USERINFO': '{"username": "cn", \
                          "phone" : "telephoneNumber", \
                          "mobile" : "mobile", \
                          "email" : "mail", \
                          "surname" : "sn", \
                          "givenname" : "givenName" }',
                      'privacyidea.ldapresolver.UIDTYPE': 'dn',
                      },
                     "")

        result = y.getUserList({'username': '*'})
        print result
        assert {'username': 'bob',
                'userid': 'cn=bob,ou=other,o=test'} in result
        
        loginId = y.getUserId("bob")

        result = y.getUserInfo(loginId)
        print result
        assert result.get("username") == "bob"
    
        result = y.checkPass("cn=alice,ou=example,o=test", "alicepw")
        print result
        assert result is True
        
        result = y.checkPass("cn=alice,ou=example,o=test", "WRONG")
        print result
        assert result is False
        
        result = y.getUsername("cn=alice,ou=example,o=test")
        print result
        assert result == "alice"
        
    def test_02_standalone_CN(self):
        '''
        Testing standalone functionality with UID Type CN
        '''
        y = getResolverClass("LDAPIdResolver", "IdResolver")()
    
        y.loadConfig({'privacyidea.ldapresolver.LDAPURI': 'ldap://localhost',
                      'privacyidea.ldapresolver.LDAPBASE': 'o=test',
                      'privacyidea.ldapresolver.BINDDN': 'cn=admin,o=test',
                      'privacyidea.ldapresolver.BINDPW': 'secret',
                      'privacyidea.ldapresolver.LOGINNAMEATTRIBUTE': 'cn',
                      'privacyidea.ldapresolver.LDAPSEARCHFILTER': '(cn=*)',
                      'privacyidea.ldapresolver.LDAPFILTER': '(&(cn=%s))',
                      'privacyidea.ldapresolver.USERINFO': '{"username": "cn", \
                          "phone" : "telephoneNumber", \
                          "mobile" : "mobile", \
                          "email" : "mail", \
                          "surname" : "sn", \
                          "givenname" : "givenName" }',
                      'privacyidea.ldapresolver.UIDTYPE': 'cn',
                      },
                     "")

        result = y.getUserList({'username': '*'})
        print result
        assert {'userid': 'bob'} in result
        
        loginId = y.getUserId("bob")

        result = y.getUserInfo(loginId)
        print result
        assert result.get("username") == "bob"
    
        result = y.checkPass("alice", "alicepw")
        print result
        assert result is True
        
        result = y.checkPass("alice", "WRONG")
        print result
        assert result is False

    def test_03_testconnection(self):
        '''
        Test the connection
        '''
        y = getResolverClass("LDAPIdResolver", "IdResolver")()
    
        y.loadConfig({'privacyidea.ldapresolver.LDAPURI': 'ldap://localhost',
                      'privacyidea.ldapresolver.LDAPBASE': 'o=test',
                      'privacyidea.ldapresolver.BINDDN': 'cn=admin,o=test',
                      'privacyidea.ldapresolver.BINDPW': 'secret',
                      'privacyidea.ldapresolver.LOGINNAMEATTRIBUTE': 'cn',
                      'privacyidea.ldapresolver.LDAPSEARCHFILTER': '(cn=*)',
                      'privacyidea.ldapresolver.LDAPFILTER': '(&(cn=%s))',
                      'privacyidea.ldapresolver.USERINFO': '{"username": "cn", \
                          "phone" : "telephoneNumber", \
                          "mobile" : "mobile", \
                          "email" : "mail", \
                          "surname" : "sn", \
                          "givenname" : "givenName" }',
                      'privacyidea.ldapresolver.UIDTYPE': 'cn',
                      },
                     "")
        
        result = y.testconnection({"BINDDN": "cn=admin,o=test",
                                   "BINDPW": "secret",
                                   "LDAPURI": "ldap://localhost",
                                   "LDAPBASE": "o=test",
                                   "LOGINNAMEATTRIBUTE": "cn",
                                   "LDAPSEARCHFILTER": "(cn=*)",
                                   'USERINFO': '{"username": "cn", \
                                      "phone" : "telephoneNumber", \
                                      "mobile" : "mobile", \
                                      "email" : "mail", \
                                      "surname" : "sn", \
                                      "givenname" : "givenName" }'
                                   })
        print result
        assert result[0] is True
        assert "Your LDAP config seems to be OK" in result[1]


# -*- coding:utf-8 -*-
from mako import runtime, filters, cache
UNDEFINED = runtime.UNDEFINED
__M_dict_builtin = dict
__M_locals_builtin = locals
_magic_number = 9
_modified_time = 1422216484.906498
_enable_loop = True
_template_filename = '/usr/lib/python2.7/dist-packages/privacyidea/lib/tokens/radiustoken.mako'
_template_uri = '/radiustoken.mako'
_source_encoding = 'utf-8'
from webhelpers.html import escape
_exports = []


def render_body(context,**pageargs):
    __M_caller = context.caller_stack._push_frame()
    try:
        __M_locals = __M_dict_builtin(pageargs=pageargs)
        c = context.get('c', UNDEFINED)
        Exception = context.get('Exception', UNDEFINED)
        _ = context.get('_', UNDEFINED)
        __M_writer = context.writer()
        # SOURCE LINE 2
        __M_writer(u'\n\n')
        # SOURCE LINE 4
        if c.scope == 'config.title' :
            # SOURCE LINE 5
            __M_writer(u' ')
            __M_writer(escape(_("RADIUS Token")))
            __M_writer(u'\n')
        # SOURCE LINE 7
        __M_writer(u'\n\n')
        # SOURCE LINE 9
        if c.scope == 'config' :
            # SOURCE LINE 10
            __M_writer(u'<script>\n\n/*\n * \'typ\'_get_config_val()\n *\n * this method is called, when the token config dialog is opened\n * - it contains the mapping of config entries to the form id\n * - according to the Config entries, the form entries will be filled\n *\n */\n\n\nfunction radius_get_config_val(){\n\tvar id_map = {};\n\n    id_map[\'radius.server\']   = \'sys_radius_server\';\n    id_map[\'radius.secret\']  = \'sys_radius_secret\';\n    id_map[\'radius.local_checkpin\'] = \'sys_radius_local_checkpin\';\n    // FIXME: We need to set the checkpin select box. Do not know how!\n\n\treturn id_map;\n\n}\n\n/*\n * \'typ\'_get_config_params()\n *\n * this method is called, when the token config is submitted\n * - it will return a hash of parameters for system/setConfig call\n *\n */\nfunction radius_get_config_params(){\n\n\tvar url_params ={};\n\n    url_params[\'radius.server\'] \t= $(\'#sys_radius_server\').val();\n    url_params[\'radius.secret\'] \t= $(\'#sys_radius_secret\').val();\n    url_params[\'radius.local_checkpin\'] \t= $(\'#sys_radius_local_checkpin\').val();\n\n\treturn url_params;\n}\n\n\njQuery.validator.addMethod("sys_radius_server", function(value, element, param){\n      return value.match(param);\n}, "')
            # SOURCE LINE 55
            __M_writer(escape(_('Please enter a valid RADIUS server specification. It needs to be of the form <name_or_ip>:<port>')))
            __M_writer(u'");\n\n\n$("#form_config_radius").validate({\n         rules: {\n            sys_radius_server: {\n                required: true,\n                number: false,\n                sys_radius_server: /^[a-z0-9.-_]*:\\d*/i\n             }\n         }\n     });\n\n</script>\n\n<form class="cmxform" id=\'form_config_radius\'>\n<fieldset>\n\t<legend>')
            # SOURCE LINE 72
            __M_writer(escape(_("RADIUS settings")))
            __M_writer(u'</legend>\n\t<table>\n\t<tr>\n\t<td><label for="sys_radius_server" title=\'')
            # SOURCE LINE 75
            __M_writer(escape(_("You need to enter the server like myradius:1812")))
            __M_writer(u"'>\n\t\t")
            # SOURCE LINE 76
            __M_writer(escape(_("RADIUS server")))
            __M_writer(u'</label></td>\n\t<td><input class="required" type="text" name="sys_radius_server" id="sys_radius_server" \n\t\tclass="text ui-widget-content ui-corner-all" value="localhost:1812"/></td>\n\t</tr>\n\n\t<tr><td><label for="sys_radius_local_checkpin" title=\'')
            # SOURCE LINE 81
            __M_writer(escape(_("The PIN can either be verified on this local privacyIDEA server or forwarded to the RADIUS server")))
            __M_writer(u"'>\n\t\t")
            # SOURCE LINE 82
            __M_writer(escape(_("check PIN")))
            __M_writer(u'</label></td>\n\t<td><select name="sys_radius_local_checkpin" id="sys_radius_local_checkpin"\n\t\ttitle=\'')
            # SOURCE LINE 84
            __M_writer(escape(_("The PIN can either be verified on this local privacyIDEA server or on the RADIUS server")))
            __M_writer(u"'>\n\t\t\t<option value=0>")
            # SOURCE LINE 85
            __M_writer(escape(_("on RADIUS server")))
            __M_writer(u'</option>\n\t\t\t<option value=1>')
            # SOURCE LINE 86
            __M_writer(escape(_("locally")))
            __M_writer(u'</option>\n\t\t</select></td>\n\t</tr><tr>\n\t<td><label for="sys_radius_secret">')
            # SOURCE LINE 89
            __M_writer(escape(_("RADIUS shared secret")))
            __M_writer(u'</label></td>\n\t<td><input type="password" name="sys_radius_secret" id="sys_radius_secret" class="text ui-widget-content ui-corner-all" /></td>\n\t</tr>\n\t</table>\n</fieldset>\n</form>\n')
        # SOURCE LINE 96
        __M_writer(u'\n\n')
        # SOURCE LINE 98
        if c.scope == 'enroll.title' :
            # SOURCE LINE 99
            __M_writer(escape(_("RADIUS token")))
            __M_writer(u'\n')
        # SOURCE LINE 101
        __M_writer(u'\n')
        # SOURCE LINE 102
        if c.scope == 'enroll' :
            # SOURCE LINE 103
            __M_writer(u'<script>\n\n/*\n * \'typ\'_get_enroll_params()\n *\n * this method is called, when the token  is submitted\n * - it will return a hash of parameters for admin/init call\n *\n */\n\nfunction radius_get_enroll_params(){\n    var params = {};\n    params[\'type\'] = \'radius\';\n\tparams[\'radius.server\'] \t\t=  $(\'#radius_server\').val();\n\tparams[\'radius.local_checkpin\'] =  $(\'#radius_local_checkpin\').val();\n\tparams[\'radius.user\'] \t\t\t=  $(\'#radius_user\').val();\n\tparams[\'radius.secret\'] \t\t=  $(\'#radius_secret\').val();\n\tparams[\'description\'] \t\t\t=  "radius:" + $(\'#radius_server\').val();\n\n\tjQuery.extend(params, add_user_data());\n    return params;\n}\n\njQuery.validator.addMethod("radius_server", function(value, element, param){\n      return value.match(param);\n}, "')
            # SOURCE LINE 128
            __M_writer(escape(_('Please enter a valid RADIUS server specification. It needs to be of the form <name_or_ip>:<port>')))
            __M_writer(u'");\n\n\n$("#form_enroll_token").validate({\n         rules: {\n            radius_server: {\n                required: true,\n                number: false,\n                radius_server: /^[a-z0-9.-_]*:\\d*/i\n             }\n         }\n     });\n\n')
            # SOURCE LINE 141

            from privacyidea.lib.config import getFromConfig
            sys_radius_server = ""
            sys_radius_secret = ""
            sys_checkpin_local = "selected"
            sys_checkpin_remote = ""
            
            try:
                    sys_radius_server = getFromConfig("radius.server")
                    sys_radius_secret = getFromConfig("radius.secret")
                    sys_radius_local_checkpin = getFromConfig("radius.local_checkpin")
            
                    if sys_radius_local_checkpin == 0:
                            sys_checkpin_local = ""
                            sys_checkpin_remote = "selected"
            except Exception:
                    pass
            
            
            
            __M_locals_builtin_stored = __M_locals_builtin()
            __M_locals.update(__M_dict_builtin([(__M_key, __M_locals_builtin_stored[__M_key]) for __M_key in ['sys_checkpin_local','getFromConfig','sys_checkpin_remote','sys_radius_secret','sys_radius_local_checkpin','sys_radius_server'] if __M_key in __M_locals_builtin_stored]))
            # SOURCE LINE 159
            __M_writer(u'\n</script>\n\n<p>')
            # SOURCE LINE 162
            __M_writer(escape(_("Here you can define, to which RADIUS server the request should be forwarded.")))
            __M_writer(u'</p>\n<p>')
            # SOURCE LINE 163
            __M_writer(escape(_("Please specify the server, the secret and the username")))
            __M_writer(u'</p>\n<table><tr>\n\t<td><label for="radius_server" title=\'')
            # SOURCE LINE 165
            __M_writer(escape(_("You need to enter the server like myradius:1812")))
            __M_writer(u"'>\n\t\t")
            # SOURCE LINE 166
            __M_writer(escape(_("RADIUS server")))
            __M_writer(u'</label></td>\n\t<td><input class="required" type="text" name="radius_server" id="radius_server" value="')
            # SOURCE LINE 167
            __M_writer(escape(sys_radius_server))
            __M_writer(u'" class="text ui-widget-content ui-corner-all"/></td>\n\t</tr><tr>\n\t<td><label for="radius_local_checkpin" title=\'')
            # SOURCE LINE 169
            __M_writer(escape(_("The PIN can either be verified on this local privacyIDEA server or forwarded to the RADIUS server")))
            __M_writer(u"'>\n\t\t")
            # SOURCE LINE 170
            __M_writer(escape(_("check PIN")))
            __M_writer(u'</label></td>\n\t<td><select name="radius_local_checkpin" id="radius_local_checkpin"\n\t\ttitle=\'')
            # SOURCE LINE 172
            __M_writer(escape(_("The PIN can either be verified on this local privacyIDEA server or on the RADIUS server")))
            __M_writer(u"'>\n\t\t\t<option ")
            # SOURCE LINE 173
            __M_writer(escape(sys_checkpin_remote))
            __M_writer(u' value=0>')
            __M_writer(escape(_("on RADIUS server")))
            __M_writer(u'</option>\n\t\t\t<option ')
            # SOURCE LINE 174
            __M_writer(escape(sys_checkpin_local))
            __M_writer(u' value=1>')
            __M_writer(escape(_("locally")))
            __M_writer(u'</option>\n\t\t</select></td>\n\t</tr><tr>\n\t<td><label for="radius_user">')
            # SOURCE LINE 177
            __M_writer(escape(_("RADIUS user")))
            __M_writer(u'</label></td>\n\t<td><input type="text" name="radius_user" id="radius_user" value="" class="text ui-widget-content ui-corner-all" /></td>\n\t</tr><tr>\n\t<td><label for="radius_secret">')
            # SOURCE LINE 180
            __M_writer(escape(_("RADIUS shared secret")))
            __M_writer(u'</label></td>\n\t<td><input type="password" name="radius_secret" id="radius_secret" value="')
            # SOURCE LINE 181
            __M_writer(escape(sys_radius_secret))
            __M_writer(u'" class="text ui-widget-content ui-corner-all" /></td>\n\t</tr></table>\n\n')
        return ''
    finally:
        context.caller_stack._pop_frame()





# -*- coding: utf-8 -*-
#  privacyIDEA is a fork of LinOTP
#  May 08, 2014 Cornelius Kölbel
#  License:  AGPLv3
#  contact:  http://www.privacyidea.org
#
#  2014-10-02 Add more tests to cover controller /gettoken
#             Cornelius Kölbel, cornelius@privacyidea.org
#
#  Copyright (C) 2010 - 2014 LSE Leading Security Experts GmbH
#  License:  AGPLv3
#  contact:  http://www.linotp.org
#            http://www.lsexperts.de
#            linotp@lsexperts.de
#
# This code is free software; you can redistribute it and/or
# modify it under the terms of the GNU AFFERO GENERAL PUBLIC LICENSE
# License as published by the Free Software Foundation; either
# version 3 of the License, or any later version.
#
# This code is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU AFFERO GENERAL PUBLIC LICENSE for more details.
#
# You should have received a copy of the GNU Affero General Public
# License along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
'''
  Description:  functional tests
                
  Dependencies: -

'''
import datetime
from json import loads

from privacyidea.tests import TestController, url

import logging
log = logging.getLogger(__name__)


class TestGetOtpController(TestController):
    '''
    This test at the moment only tests the implementation for the Tagespasswort
    Token
    
        getotp
        get_multi_otp
    '''
    seed = "3132333435363738393031323334353637383930"
    seed32 = "3132333435363738393031323334353637383930313233343536373839303132"
    seed64 = "31323334353637383930313233343536373839303132333435363738393031"\
             "32333435363738393031323334353637383930313233343536373839303132"\
             "3334"

    def setUp(self):
        '''
        This sets up all the resolvers and realms
        '''
        TestController.setUp(self)
        self.curTime = datetime.datetime(2012, 5, 16, 9, 0, 52, 227413)
        # self.TOTPcurTime = datetime.datetime.fromtimestamp(1337292860.585256)
        self.TOTPTimestamp = 1337292860
        self.initToken()

    def createDPWToken(self, serial, seed):
        '''
        creates the test tokens
        '''
        parameters = {"serial": serial,
                      "type": "DPW",
                      # 64 byte key
                      "otpkey": seed,
                      "otppin": "1234",
                      "pin": "pin",
                      "otplen": 6,
                      "description": "DPW testtoken",
                      }

        response = self.app.get(url(controller='admin', action='init'),
                                params=parameters)
        assert '"value": true' in response

    def createHOTPToken(self, serial, seed):
        '''
        creates the test tokens
        '''
        parameters = {"serial": serial,
                      "type": "HMAC",
                      # 64 byte key
                      "otpkey": seed,
                      "otppin": "1234",
                      "pin": "pin",
                      "otplen": 6,
                      "description": "HOTP testtoken",
                      }

        response = self.app.get(url(controller='admin', action='init'),
                                params=parameters)
        assert '"value": true' in response

    def createTOTPToken(self, serial, seed, timeStep=30):
        '''
        creates the test tokens
        '''
        parameters = {"serial": serial,
                      "type": "TOTP",
                      # 64 byte key
                      "otpkey": seed,
                      "otppin": "1234",
                      "pin": "pin",
                      "otplen": 8,
                      "description": "TOTP testtoken",
                      "timeStep": timeStep,
                      }

        response = self.app.get(url(controller='admin', action='init'),
                                params=parameters)
        assert '"value": true' in response

    def removeTokenBySerial(self, serial):

        parameters = {"serial": serial,
                      }

        response = self.app.get(url(controller='admin', action='remove'),
                                params=parameters)
        return response

    def setTokenRealm(self, serial, realms):
        parameters = {"serial": serial,
                      "realms": realms
                      }

        response = self.app.get(url(controller="admin", action="tokenrealm"),
                                params=parameters)
        return response

    def initToken(self):
        '''
        init one DPW token
        '''

        self.createDPWToken("dpw1", "12341234123412341234123412341234")
        '''
        curTime = datetime.datetime(2012, 5, 16, 9, 0, 52, 227413)
            "12-05-22": "202690",
            "12-05-23": "252315",
            "12-05-20": "6",
            "12-05-21": "325819",
            "12-05-24": "263973",
            "12-05-25": "321965",
            "12-05-17": "028193",
            "12-05-16": "427701",
            "12-05-19": "167074",
            "12-05-18": "857788"
        '''
        self.createHOTPToken("hotp1", "12341234123412341234123412341234")
        '''
            "0": "819132",
            "1": "301156",
            "2": "586172",
            "3": "720026",
            "4": "062511",
            "5": "598723",
            "6": "770725",
            "7": "596337",
            "8": "647211",
            "9": "294016",
            "10": "161051",
            "11": "886458"
        '''
        self.createHOTPToken("hotp2", "12341234123412341234123412341234")
        '''
            "0": "819132",
            "1": "301156",
            "2": "586172",
            "3": "720026",
            "4": "062511",
            "5": "598723",
            "6": "770725",
            "7": "596337",
            "8": "647211",
            "9": "294016",
            "10": "161051",
            "11": "886458"
        '''
        self.createTOTPToken("totp1", self.seed, timeStep=30)
        '''
        T0=44576428.686175205 (*30)
            "44576428": "33726427",
            "44576429": "84341529",
            "44576430": "35692495",
            "44576431": "70995873",
            "44576432": "12048114",
            "44576433": "06245460",
            "44576434": "10441015",
            "44576435": "50389782",
            "44576436": "78905052",
            "44576437": "52978758",
            "44576438": "90386435",
            "44576439": "76892112"
        
        '''

        resp = self.setTokenRealm("dpw1", "mydefrealm")
        print resp
        assert '"status": true' in resp

        resp = self.setTokenRealm("hotp1", "mydefrealm")
        print resp
        assert '"status": true' in resp

        resp = self.setTokenRealm("totp1", "mydefrealm")
        print resp
        assert '"status": true' in resp

        self.unassign("totp1")
        resp = self.app.get(url(controller='admin', action='assign'),
                            {'user': 'localuser',
                             'serial': 'totp1'
                             })
        assert '"status": true' in resp

        # parameters = {'user' : 'root',
        #              'serial' : 'totp1' }
        parameters = {}
        resp = self.app.get(url(controller='system', action='getRealms'),
                            params=parameters)
        print "getRealms: ", resp
        assert '"status": true' in resp

        parameters = {'name': 'getmultitoken',
                      'scope': 'gettoken',
                      'realm': 'mydefrealm',
                      'action': 'max_count_dpw=10, max_count_hotp=10, '
                      'max_count_totp=10',
                      'user': 'admin'
                      }
        response = self.app.get(url(controller='system', action='setPolicy'),
                                params=parameters)
        print "setPolicy:", response
        assert '"status": true' in response
        
        response = self.app.get(url(controller='system', action='getPolicy'),
                                params=parameters)
        print "getPolicy:", response
        assert '"getmultitoken": {' in response

        response = self.app.get(url(controller='system', action='getConfig'),
                                params={})
        print "config", response
        assert '"status": true' in response
        
    def test_01_getotp_dpw(self):
        '''
        get otp value of the DPW token
        '''
        parameters = {'serial': 'dpw1',
                      'curTime': self.curTime,
                      'selftest_admin': 'admin'
                      }
        response = self.app.get(url(controller='gettoken',
                                    action='getotp'),
                                params=parameters)
        print "current time %s" % self.curTime
        print response
        assert '"otpval": "427701"' in response
        
    def test_02_getotp_policyfail(self):
        '''
        get otp fails due to policy
        '''
        parameters = {'serial': 'dpw1',
                      'curTime': self.curTime,
                      'selftest_admin': 'admin2'
                      }
        response = self.app.get(url(controller='gettoken',
                                    action='getotp'),
                                params=parameters)
        print "current time %s" % self.curTime
        print response
        assert '"status": false' in response
        assert 'The policy forbids receiving OTP values' in response

    def test_03_getmultiotp(self):
        '''
        get otp value of the DPW token
        '''
        parameters = {'serial': 'dpw1',
                      'curTime': self.curTime,
                      'count': "10",
                      'selftest_admin': 'admin'
                      }
        response = self.app.get(url(controller='gettoken',
                                    action='getmultiotp'),
                                params=parameters)
        print response
        assert '"12-05-17": "028193"' in response
        assert '"12-05-18": "857788"' in response
        
        parameters = {'serial': 'dpw1',
                      'curTime': self.curTime,
                      'count': "10",
                      'selftest_admin': 'admin',
                      'view': '1'
                      }
        response = self.app.get(url(controller='gettoken',
                                    action='getmultiotp'),
                                params=parameters)
        print response
        assert '<td class="getotp key">12-05-16</td>' in response
        assert '<td class="getotp value">427701</td>' in response
        
    def test_04_getmultiotp(self):
        '''
        get multi otp fails due to policy
        '''
        parameters = {'serial': 'dpw1',
                      'curTime': self.curTime,
                      'count': "10",
                      'selftest_admin': 'looseradmin'
                      }
        response = self.app.get(url(controller='gettoken',
                                    action='getmultiotp'),
                                params=parameters)
        print response
        assert '"result": false' in response
        assert 'No count specified' in response

    def test_05_getotp_hotp(self):
        '''
        get otp value of the HOTP token
        '''
        parameters = {'serial': 'hotp1'}
        response = self.app.get(url(controller='gettoken',
                                    action='getotp'),
                                params=parameters)
        print response
        assert '"otpval": "819132"' in response
        
    def test_05a_gettoken_by_user(self):
        '''
        get otp value by user
        '''
        response = self.app.get(url(controller='admin',
                                    action='assign'),
                                params={'serial': 'hotp1',
                                        'user': 'user1'})
        print response
        assert '"value": true' in response
    
        # getotp by user
        response = self.app.get(url(controller='gettoken',
                                    action='getotp'),
                                params={'user': 'user1'})
        print response
        assert '"otpval": "819132"' in response
        
        # assign the second token to the user
        response = self.app.get(url(controller='admin',
                                    action='assign'),
                                params={'serial': 'dpw1',
                                        'user': 'user1'})
        print response
        assert '"value": true' in response
        
        response = self.app.get(url(controller='gettoken',
                                    action='getotp'),
                                params={'user': 'user1'})
        print response
        assert '"result": false' in response
        
        # clean up: unassign token
        for serial in ["dpw1", "hotp1"]:
            response = self.app.get(url(controller='admin',
                                        action='unassign'),
                                    params={'serial': serial})
            print response
            assert '"value": true' in response
            
    def test_05b_gettoken_for_wildcard(self):
        '''
        get otp for a user with wildcard policy
        '''
        parameters = {'name': 'getalluser',
                      'scope': 'gettoken',
                      'realm': 'mydefrealm',
                      'action': 'max_count_hotp=1'
                      }
        response = self.app.get(url(controller='system', action='setPolicy'),
                                params=parameters)
        print response
        assert '"status": true' in response
        
        response = self.app.get(url(controller='admin', action='assign'),
                                params={"serial": "hotp2",
                                        "user": "horst"})
        print response
        assert '"status": true' in response
        
        response = self.app.get(url(controller="gettoken", action="getotp"),
                                params={"serial": "hotp2",
                                        "user": "horst2",
                                        "realm": "mydefrealm"})
        print response
        assert '"otpval": "819132",' in response
        
        response = self.app.get(url(controller='admin', action='unassign'),
                                params={"serial": "hotp2"})
        print response
        assert '"status": true' in response
        
        response = self.app.get(url(controller='system', action='delPolicy'),
                                params={"name": "getalluser"})
        print response
        assert '"status": true' in response
            
    def test_00_missing_parameter(self):
        '''
        get otp missing parameter
        '''
        response = self.app.get(url(controller='gettoken',
                                    action='getotp'),
                                params={})
        print response
        assert('"description": "you need to provide a user or a serial"'
               in response)
        
    def test_00_user_without_token(self):
        '''
        get otp from user without token
        '''
        response = self.app.get(url(controller='gettoken',
                                    action='getotp'),
                                params={'user': 'horst'})
        print response
        assert('"description": "No Token found for this user"'
               in response)
        
    def test_00_wrong_serial(self):
        '''
        get otp from wrong serial
        '''
        response = self.app.get(url(controller='gettoken',
                                    action='getotp'),
                                params={'serial': '123XXX'})
        print response
        assert('ERR1119: getTokenRealms failed. No token with serial '
               '123XXX found' in response)
        
    def test_00_token_without_getotp(self):
        '''
        get otp from token, not supporting getOtp
        '''
        response = self.app.get(url(controller='admin', action='init'),
                                params={'serial': 'spass1',
                                        'type': 'spass'})
        print response
        assert '"status": true' in response
        assert '"value": true' in response

        response = self.app.get(url(controller='gettoken',
                                    action='getotp'),
                                params={'serial': 'spass1'})
        print response
        assert('ERR410: There is no policy gettoken/max_count definable '
               'for the tokentype' in response)
        
        response = self.app.get(url(controller='admin', action='remove'),
                                params={'serial': 'spass1'})
        print response
        assert '"status": true' in response
        assert '"value": 1' in response

    def test_06_getmultiotp(self):
        '''
        get otp value of the HOTP token
        '''
        parameters = {'serial': 'hotp1',
                      'curTime': self.curTime,
                      'count': "20",
                      'selftest_admin': 'admin'
                      }
        response = self.app.get(url(controller='gettoken',
                                    action='getmultiotp'),
                                params=parameters)
        print response
        assert '"0": "819132"' in response
        assert '"1": "301156"' in response

    def test_07_getotp_totp(self):
        '''
        get otp value of the TOTP token
        
        
          +-------------+--------------+------------------+----------+--------+
          |  Time (sec) |   UTC Time   | Value of T (hex) |   TOTP   |  Mode  |
          +-------------+--------------+------------------+----------+--------+
          |      59     |  1970-01-01  | 0000000000000001 | 94287082 |  SHA1  |
          |             |   00:00:59   |                  |          |        |
          |  1111111109 |  2005-03-18  | 00000000023523EC | 07081804 |  SHA1  |
          1111107509
          |             |   01:58:29   |                  |          |        |
          |  1111111111 |  2005-03-18  | 00000000023523ED | 14050471 |  SHA1  |
          |             |   01:58:31   |                  |          |        |
          |  1234567890 |  2009-02-13  | 000000000273EF07 | 89005924 |  SHA1  |
          |             |   23:31:30   |                  |          |        |
          |  2000000000 |  2033-05-18  | 0000000003F940AA | 69279037 |  SHA1  |
          |             |   03:33:20   |                  |          |        |
          | 20000000000 |  2603-10-11  | 0000000027BC86AA | 65353130 |  SHA1  |
          |             |   11:33:20   |                  |          |        |
        '''
        cTimes = [('1970-01-01 00:00:59', '94287082'),
                  ('2005-03-18 01:58:29', '07081804'),
                  ('2005-03-18 01:58:31', '14050471'),
                  ('2009-02-13 23:31:30', '89005924'),
                  ('2033-05-18 03:33:20', '69279037'),
                  ]
        for cTime in cTimes:
            TOTPcurTime = cTime[0]
            otp = cTime[1]

            parameters = {'serial': 'totp1',
                          'curTime': TOTPcurTime}
            response = self.app.get(url(controller='gettoken',
                                        action='getotp'),
                                    params=parameters)
            print response
            assert otp in response

        return

    def test_08_getmultiotp(self):
        '''
        get otp value of the TOTP token
        '''
        parameters = {'serial': 'totp1',
                      'timestamp': self.TOTPTimestamp,
                      'count': "20",
                      'selftest_admin': 'admin'}
        response = self.app.get(url(controller='gettoken',
                                    action='getmultiotp'),
                                params=parameters)
        print response
        resp = loads(response.body)
        otps = resp.get('result').get('value').get('otp')

        otp1 = otps.get('44576429')
        assert otp1.get('otpval') == '36163821'

        otp2 = otps.get('44576430')
        assert otp2.get('otpval') == '58711820'

        return

    def test_09_usergetmultiotp_no_policy(self):
        '''
        get otp value for a users own token with missing policy
        '''
        parameters = {'serial': 'totp1',
                      'timestamp': self.TOTPTimestamp,
                      'count': "20",
                      'selftest_user': 'localuser@mydefrealm'}
        response = self.app.get(url(controller='selfservice',
                                    action='usergetmultiotp'),
                                params=parameters)
        print "test_09: ", response
        assert '"message": "ERR410:' in response

    def test_10_usergetmultiotp(self):
        '''
        get otp value for a users own token
        '''
        parameters = {'name': 'usertoken',
                      'scope': 'selfservice',
                      'realm': 'mydefrealm',
                      'action': 'max_count_dpw=10, max_count_hotp=10, '
                      'max_count_totp=10'
                      }
        response = self.app.get(url(controller='system', action='setPolicy'),
                                params=parameters)
        print "setPolicy:", response
        assert '"status": true' in response

        parameters = {'serial': 'totp1',
                      'timestamp': self.TOTPTimestamp,
                      'count': "20",
                      'selftest_user': 'localuser@mydefrealm'}
        response = self.app.get(url(controller='selfservice',
                                    action='usergetmultiotp'),
                                params=parameters)
        print response

        resp = loads(response.body)
        otps = resp.get('result').get('value').get('otp')

        otp1 = otps.get('44576428')
        assert otp1.get('otpval') == '85291609'

        otp2 = otps.get('44576437')
        assert otp2.get('otpval') == '74602968'

        return

    def test_11_usergetmultiotp_fail(self):
        '''
        get otp value for a  token that does not belong to the user
        '''
        parameters = {'serial': 'hotp1',
                      'timestamp': self.TOTPTimestamp,
                      'count': "20",
                      'selftest_user': 'localuser@mydefrealm'}
        response = self.app.get(url(controller='selfservice',
                                    action='usergetmultiotp'),
                                params=parameters)
        print response
        assert('"message": "The serial hotp1 does not belong to user '
               'localuser@mydefrealm"' in response)



# -*- coding:utf-8 -*-
from mako import runtime, filters, cache
UNDEFINED = runtime.UNDEFINED
__M_dict_builtin = dict
__M_locals_builtin = locals
_magic_number = 9
_modified_time = 1422216555.35042
_enable_loop = True
_template_filename = '/usr/lib/python2.7/dist-packages/privacyidea/templates/manage/userview.mako'
_template_uri = '/manage/userview.mako'
_source_encoding = 'utf-8'
from webhelpers.html import escape
_exports = []


def render_body(context,**pageargs):
    __M_caller = context.caller_stack._push_frame()
    try:
        __M_locals = __M_dict_builtin(pageargs=pageargs)
        c = context.get('c', UNDEFINED)
        _ = context.get('_', UNDEFINED)
        __M_writer = context.writer()
        # SOURCE LINE 2
        __M_writer(u'<div style="float:right">\n<a href=\'')
        # SOURCE LINE 3
        __M_writer(escape(c.help_url))
        __M_writer(u'/userview/index.html\' target="_blank">\n\t<img alt="(?)" width=24\n\tsrc="/images/help32.png"  \n\ttitle=\'')
        # SOURCE LINE 6
        __M_writer(escape(_("help on userview")))
        __M_writer(u'\'>\n</a>\n</div>\n\n<table id="user_table" class="flexme2" style="display:none"></table>\n   \n<script type="text/javascript"> \n\nview_user();\n\n</script>\n')
        return ''
    finally:
        context.caller_stack._pop_frame()





# -*- coding: utf-8 -*-
#
#    privacyIDEA is a fork of LinOTP
#    May 28, 2014 Cornelius Kölbel
#    E-mail: info@privacyidea.org
#    Contact: www.privacyidea.org
#
#    Copyright (C) LinOTP: 2010 - 2014 LSE Leading Security Experts GmbH
#
#    This program is free software: you can redistribute it and/or
#    modify it under the terms of the GNU Affero General Public
#    License, version 3, as published by the Free Software Foundation.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the
#               GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
#

"""
This is the SMSClass to send SMS via HTTP Gateways
It can handle HTTP/HTTPS PUT and GET requests also with Proxy support
"""

#from SMSProvider import getSMSProviderClass
from privacyidea.smsprovider.SMSProvider import ISMSProvider

import base64

import urllib
import httplib2
import urllib2
from urlparse import urlparse

import logging
log = logging.getLogger(__name__)

## on debian squeeze the httplib is too old and does not contain
## a socks module. So we take an elder one, which does satiisfy
## the import BUT it does not work as well as the former urllib
## proxy does not work :-(

try:
    import httplib2.socks as socks
    log.info('Using httplib2.socks')
except ImportError:
    import socks as socks
    log.info('Using socksipy socks')


class HttpSMSProvider(ISMSProvider):
    def __init__(self):
        self.config = {}

    def submitMessage(self, phone, message):
        '''
        send out a message to a phone via an http sms connector
        :param phone: the phone number
        :param message: the message to submit to the phone
        '''
        url = self.config.get('URL', None)
        if url is None:
            return

        log.debug("[submitMessage] submitting message "
                  "%s to %s" % (message, phone))

        method = self.config.get('HTTP_Method', 'GET')
        username = self.config.get('USERNAME', None)
        password = self.config.get('PASSWORD', None)

        log.debug("[submitMessage] by method %s" % method)
        parameter = self.getParameters(message, phone)

        log.debug("[submitMessage] Now doing the Request")

        ## urlib2 has problems with authentication AND https
        ## below a test of urllib and httplib which shows, that
        ## we should use in case of Basic Auth and https the httplib:

        #NO_PROX  --  HTTPS Basic Auth  -- urllib  -- : Fail
        #NO_PROX  --  HTTPS  --            urllib  -- : Ok
        #NO_PROX  --  HTTP Basic Auth  --  urllib  -- : Ok
        #NO_PROX  --  HTTP  --             urllib  -- : Ok

        #PROX  --     HTTPS Basic Auth  -- urllib  -- : Fail
        #PROX  --     HTTPS  --            urllib  -- : Ok
        #PROX  --     HTTP Basic Auth  --  urllib  -- : Ok
        #PROX  --     HTTP  --             urllib  -- : Ok


        #NO_PROX  -- HTTPS Basic Auth  --  httplib  -- : OK
        #NO_PROX  -- HTTPS  --             httplib  -- : OK
        #NO_PROX  -- HTTP Basic Auth  --   httplib  -- : OK
        #NO_PROX  -- HTTP  --              httplib  -- : OK

        #PROX  --    HTTPS Basic Auth  -- httplib  -- : OK
        #PROX  --    HTTPS  --            httplib  -- : OK
        #PROX  --    HTTP Basic Auth  --  httplib  -- : Fail
        #PROX  --    HTTP  --             httplib  -- : Fail


        basic_auth = False
        https = False

        ## there might be the basic authentication in the request url
        ## like http://user:passw@hostname:port/path
        if password is None and username is None:
            parsed_url = urlparse(url)
            if "@" in parsed_url[1]:
                puser, server = parsed_url[1].split('@')
                username, password = puser.split(':')

        if username and password is not None:
            basic_auth = True

        if url.startswith('https:'):
            https = True

        lib = 'urllib'
        try:
            if basic_auth == True and https == True:
                lib = 'httplib'
                ret = self.httplib_request(url, parameter, username=username,
                                           password=password, method=method)
            else:
                ret = self.urllib_request(url, parameter, username=username,
                                           password=password, method=method)
        except Exception as exx:

            log.warning("Failed to access the HTTP SMS Service with %s: %r"
                        % (lib, exx))
            try:
                if lib == 'httplib':
                    ret = self.urllib_request(url, parameter, username=username,
                                               password=password, method=method)
                else:
                    ret = self.httplib_request(url, parameter, username=username,
                                               password=password, method=method)
            except Exception as new_exx:
                ## if we as well get an error, we raise the first exception
                ## to be more authentic ;-)
                log.warning("Failed again to access the HTTP SMS Service: %r"
                        % new_exx)
                raise exx


        return ret

    def getParameters(self, message, phone):

        urldata = {}

        ## transfer the phone key
        phoneKey = self.config.get('SMS_PHONENUMBER_KEY', "phone")
        urldata[phoneKey] = phone
        log.debug("[getParameters] urldata: %s" % urldata)

        ## transfer the sms key
        messageKey = self.config.get('SMS_TEXT_KEY', "sms")
        urldata[messageKey] = message
        log.debug("[getParameters] urldata: %s" % urldata)

        params = self.config.get('PARAMETER', {})
        urldata.update(params)

        log.debug("[getParameters] urldata: %s" % urldata)

        return urldata


    def _check_success(self, reply):
        '''
        Check the success according to the reply
        1. if RETURN_SUCCESS is defined
        2. if RETURN_FAIL is defined
        '''
        ret = False
        log.debug("[_check_success] entering with config %s" % self.config)
        if "RETURN_SUCCESS" in self.config:
            success = self.config.get("RETURN_SUCCESS")
            log.debug("[_check_success] success: %s" % success)
            if reply[:len(success)] == success:
                log.debug("[_check_success] sending sms success")
                ret = True
            else:
                log.warning("[_check_success] failed to send sms. Reply does not match the RETURN_SUCCESS definition")
                raise Exception("We received a none success reply from the SMS Gateway.")

        elif "RETURN_FAIL" in self.config:
            fail = self.config.get("RETURN_FAIL")
            log.debug("[_check_success] fail: %s" % fail)
            if reply[:len(fail)] == fail:
                log.warning("[_check_success] sending sms fail")
                raise Exception("We received a predefined error from the SMS Gateway.")
            else:
                log.warning("[_check_success] sending sms success full. The reply does not match the RETURN_FAIL definition")
                ret = True
        else:
            ret = True
        return ret

    def get_proxy_info(self, proxy):
        """
        helper to parse the proxyurl and to create the proxy_info object

        :param proxy: proxy url string
        :return: ProxyInfo object
        """
        proxy_params = {}
        proxy_host = None
        proxy_port = 8888

        parts = urlparse(proxy)
        net_loc = parts[1]

        if "@" in net_loc:
            puser, server = net_loc.split('@')
            if ':' in puser:
                proxy_user, proxy_pass = puser.split(':')
                proxy_params["proxy_user"] = proxy_user
                proxy_params["proxy_pass"] = proxy_pass
        else:
            server = net_loc

        if ':' in server:
            proxy_host, port = server.split(':')
            proxy_port = int(port)
        else:
            proxy_host = server

        # using httplib2:
        # the proxy spec and url + enc. parameters must be of
        # type string str() - otherwise the following error will occur:
        # : GeneralProxyError: (5, 'bad input') :

        proxy_info = httplib2.ProxyInfo(proxy_type=socks.PROXY_TYPE_HTTP,
                                        proxy_host=proxy_host,
                                        proxy_port=proxy_port,
                                        **proxy_params)
        return proxy_info

    def httplib_request(self, url, parameter,
                       username=None, password=None, method='GET'):
        """
        build the urllib request and check the response for success or fail



        :param url: target url
        :param parameter: additonal parameter to append to the url request
        :param username: basic authentication with username (optional)
        :param password: basic authentication with password (optional)
        :param method: run an GET or POST request

        :return: False or True
        """

        #httplib2.debuglevel = 4

        ret = False
        http_params = {}

        log.debug("Do the request to %s with %s" % (url, parameter))

        if 'PROXY' in self.config:
            ## prepare proxy from urls like
            ##   "http://username:password@your-proxy:8080"
            proxy = str(self.config['PROXY'])
            proxy_info = self.get_proxy_info(proxy)
            http_params["proxy_info"] = proxy_info

        http_params["disable_ssl_certificate_validation"] = True

        try:
            ## test if httplib is compiled with ssl - will raise a TypeError
            ## TypeError: __init__() got an unexpected keyword argument
            ## 'disable_ssl_certificate_validation'
            http = httplib2.Http(**http_params)
        except TypeError as exx:
            log.warning("httplib2 'disable_ssl_certificate_validation' "
                        "attribute error: %r" % exx)
            ## so we remove the ssl param from the arguments
            del http_params["disable_ssl_certificate_validation"]
            ## and retry
            http = httplib2.Http(**http_params)

        ## for backward compatibility we have to support url with the format
        ##  http://user:pass@server:port/path
        ## so we extract the url_user and the url_pass and use them if
        ## not overruled by the explicit parameters username and password
        url_user = None
        url_pass = None
        parsed_url = urlparse(url)

        if "@" in parsed_url[1]:
            puser, server = parsed_url[1].split('@')
            url_user, url_pass = puser.split(':')

            ## now rewrite the url to not contain the user anymore
            url = url.replace(parsed_url[1], server)

        if username and password is not None:
            http.add_credentials(name=username, password=password)
        elif url_user and url_pass is not None:
            http.add_credentials(name=url_user, password=url_pass)

        ##! the parameters to the httplib / proxy must be of type str()
        encoded_params = str(urllib.urlencode(parameter))
        call_url = str(url)

        try:
            ## do a GET request - which has no body but all params
            ## added to the url
            if method == 'GET':
                call_data = None
                if len(encoded_params) > 0:
                    ## extend the url with our parameters
                    call_url = str("%s?%s" % (call_url, encoded_params))

            ## or do a POST request - the more secure default and fallback
            else:
                method = 'POST'
                call_data = encoded_params

            # using httplib2:
            # the proxy spec and url + enc. parameters must be of
            # type string str() - otherwise the following error will occur:
            # : GeneralProxyError: (5, 'bad input') :

            (_resp, reply) = http.request(call_url, method=method,
                                         body=call_data)

            # some providers like clickatell have no response.status!
            log.debug("HttpSMSProvider >>%s...%s<<", reply[:20], reply[-20:])
            ret = self._check_success(reply)

        except Exception as exc:
            log.error("HttpSMSProvider %r" % exc)
            raise Exception("Failed to send SMS. %s" % str(exc))

        return ret


    def urllib_request(self, url, parameter,
                       username=None, password=None, method='GET'):
        """
        build the urllib request and check the response for success or fail

        :param url: target url
        :param parameter: additonal parameter to append to the url request
        :param username: basic authentication with username (optional)
        :param password: basic authentication with password (optional)
        :param method: run an GET or POST request

        :return: False or True
        """
        try:

            handlers = []
            if 'PROXY' in self.config and self.config['PROXY']:
                # for simplicity we set both protocols
                proxy_handler = urllib2.ProxyHandler({"http": self.config['PROXY'],
                                                      "https":self.config['PROXY']})
                handlers.append(proxy_handler)
                print "using Proxy: %r" % self.config['PROXY']

            if username and password is not None:
                password_mgr = urllib2.HTTPPasswordMgrWithDefaultRealm()
                password_mgr.add_password(None, url, username, password)
                auth_handler = urllib2.HTTPBasicAuthHandler(password_mgr)
                handlers.append(auth_handler)

            opener = urllib2.build_opener(*handlers)
            urllib2.install_opener(opener)

            full_url = str(url)
            encoded_params = None
            if parameter is not None and len(parameter) > 0 :
                encoded_params = str(urllib.urlencode(parameter))

            if method == 'GET':
                c_data = None
                if encoded_params:
                    full_url = str("%s?%s" % (url, encoded_params))
            else:
                c_data = encoded_params

            requ = urllib2.Request(full_url, data=c_data, headers={})
            if username and password is not None:
                base64string = base64.encodestring('%s:%s' % (username, password)).replace('\n', '')
                requ.add_header("Authorization", "Basic %s" % base64string)

            response = urllib2.urlopen(requ)
            reply = response.read()

            # some providers like clickatell have no response.status!
            log.debug("HttpSMSProvider >>%s...%s<<", reply[:20], reply[-20:])
            ret = self._check_success(reply)

        except Exception as exc:
            log.error("HttpSMSProvider %r" % exc)
            raise Exception("Failed to send SMS. %s" % str(exc))

        return ret


    def loadConfig(self, configDict):
        self.config = configDict

##eof##########################################################################



# -*- coding: utf-8 -*-
#
#    privacyIDEA Account test suite
#
#    Copyright (C)  2014 Cornelius Kölbel, cornelius@privacyidea.org
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.

import logging
from privacyidea.tests import TestController, url
from webtest.app import AppError

log = logging.getLogger(__name__)


class TestAccountController(TestController):

    def test_login(self):
        try:
            response = self.app.get(url(controller='account',
                                        action='login'),
                                    params={"login": "user",
                                            "password": "password"})
            print response
        except AppError as ae:
            self.assertTrue('576 Logout from privacyIDEA selfservice'
                            in u"%r" % ae)


# -*- coding: utf-8 -*-
'''
  product:  privacyIDEA
  module:   privacyidea weblib
    
  Copyright (c) 2014 Cornelius Kölbel
  License:  AGPLv3
  contact:  http://www.privacyidea.org
            
  Description:  The weblib is supposed to contain base functions that
                will only work with pylons

  Dependencies: -
'''


# -*- coding: utf-8 -*-
#  privacyIDEA is a fork of LinOTP
#  May 08, 2014 Cornelius Kölbel
#  License:  AGPLv3
#  contact:  http://www.privacyidea.org
#
#  2014 Sep 29 Cornelius Kölbel, cornelius@privacyidea.org
#              added tests for getRealmsOfTokenOrUser and getAllTokenUsers
#
#  Copyright (C) 2010 - 2014 LSE Leading Security Experts GmbH
#  License:  AGPLv3
#  contact:  http://www.linotp.org
#            http://www.lsexperts.de
#            linotp@lsexperts.de
#
# This code is free software; you can redistribute it and/or
# modify it under the terms of the GNU AFFERO GENERAL PUBLIC LICENSE
# License as published by the Free Software Foundation; either
# version 3 of the License, or any later version.
#
# This code is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU AFFERO GENERAL PUBLIC LICENSE for more details.
#
# You should have received a copy of the GNU Affero General Public
# License along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
'''     
  Description:  functional tests
                
  Dependencies: -

'''
import logging
import pylons
from paste.deploy import loadapp
from webtest import TestApp

from privacyidea.tests import TestController, url

log = logging.getLogger(__name__)


class TestManageController(TestController):

    ###############################################################################
    def test_01_index(self):
        '''
        Manage: testing index access
        '''
        response = self.app.get(url(controller='manage', action='index'),
                                params={})
        print "index response: %r" % response.testbody
        assert '<title>privacyIDEA Management</title>' in response.testbody


    def test_02_policies(self):
        '''
        Manage: testing policies tab
        '''
        response = self.app.get(url(controller='manage', action='policies'),
                                params={})
        print "policies response: %r" % response.testbody
        assert '<a id=policy_export>' in response.testbody
        assert '<button id=policy_import>' in response.testbody
        assert '<button  id=button_policy_delete>' in response.testbody

    def test_03_audit(self):
        '''
        Manage: testing audit trail
        '''
        response = self.app.get(url(controller='manage', action='audittrail'),
                                params={})
        print "audit response: %r" % response.testbody
        assert 'table id="audit_table"' in response.testbody
        assert 'view_audit();' in response.testbody

    def test_04_tokenview(self):
        '''
        Manage: testing tokenview
        '''
        response = self.app.get(url(controller='manage', action='tokenview'),
                                params={})
        print "token response: %r" % response.testbody
        assert 'button_losttoken' in response.testbody
        assert 'button_tokeninfo' in response.testbody
        assert 'button_resync' in response.testbody
        assert 'button_tokenrealm' in response.testbody
        assert 'table id="token_table"' in response.testbody
        assert 'view_token();' in response.testbody
        assert 'tokenbuttons();' in response.testbody

    def test_05_userview(self):
        '''
        Manage: testing userview
        '''
        response = self.app.get(url(controller='manage', action='userview'),
                                params={})
        print "user response: %r" % response.testbody
        assert 'table id="user_table"' in response.testbody
        assert 'view_user();' in response.testbody

    def test_06_tokenflexi(self):
        '''
        Manage: testing the tokenview_flexi method
        '''
        response = self.app.get(url(controller='manage', action='tokenview_flexi'),
                                params={})

        testbody = response.body.replace('\n', ' ').replace('\r', '').replace("  ", " ")
        print "token flexi response 1: %r" % response.testbody
        #assert '"total": 3' in testbody
        assert '"token1",       true,       "heinz"' in testbody
        assert '"token2",       true,       "nick"' in testbody
        assert '"token3",       true,       "renate"' in testbody

        # only renates token
        response = self.app.get(url(controller='manage', action='tokenview_flexi'),
                                params={'qtype' : 'loginname',
                                        'query' : 'renate'})
        testbody = response.body.replace('\n', ' ').replace('\r', '').replace("  ", " ")
        print "token flexi response 2: %r" % response.testbody
        assert '"total": 1' in testbody
        assert '"token3",       true,       "renate"' in testbody

        # only tokens in realm1
        response = self.app.get(url(controller='manage', action='tokenview_flexi'),
                                params={'qtype' : 'realm',
                                        'query' : 'realm1'})
        print "token flexi response 3: %r" % response.testbody
        assert '"total": 2' in response.testbody
        testbody = response.body.replace('\n', ' ').replace('\r', '').replace("  ", " ")
        assert '"token1",       true,       "heinz"' in testbody
        assert '"token2",       true,       "nick"' in testbody

        # search in all columns
        response = self.app.get(url(controller='manage', action='tokenview_flexi'),
                                params={'qtype' : 'all',
                                        'query' : 'token2'})
        print "token flexi response 4: %r" % response.testbody
        assert '"total": 1' in response.testbody
        testbody = response.body.replace('\n', ' ').replace('\r', '').replace("  ", " ")
        assert '"token2",       true,       "nick"' in testbody

        # with wildcard
        response = self.app.get(url(controller='manage', action='tokenview_flexi'),
                                params={"qtype": "loginname",
                                        "query": "r*"})
        testbody = response.body.replace('\n', ' ').replace('\r', '').replace("  ", " ")
        print "=== Testing with wildcard ==="
        print testbody
        assert '"total": 1' in testbody
        assert 'token3' in testbody

    def test_07_userflexi(self):
        '''
        Manage: testing the userview_flexi method
        '''
        # No realm, no user
        response = self.app.get(url(controller='manage', action='userview_flexi'),
                                params={})
        print "user flexi response 1: %r" % response.testbody
        assert '"total": 0' in response.testbody

        # No realm, no user

        response = self.app.get(url(controller='manage', action='userview_flexi'),
                                params={"page" :1,
                                        "rp": 15,
                                        "sortname": "username",
                                        "sortorder": "asc",
                                        "query": "",
                                        "qtype": "username",
                                        "realm": "realm1"})
        print "user flexi response 2: %r" % response.testbody
        assert '"id": "heinz"' in response.testbody


        response = self.app.get(url(controller='manage', action='userview_flexi'),
                                params={"page" :1,
                                        "rp": 15,
                                        "sortname": "username",
                                        "sortorder": "desc",
                                        "query": "",
                                        "qtype": "username",
                                        "realm": "realm2"})
        print "user flexi response 3: %r" % response.testbody
        assert '"id": "renate"' in response.testbody


    def test_08_tokeninfo(self):
        '''
        Manage: Testing tokeninfo dialog
        '''
        response = self.app.get(url(controller='manage', action='tokeninfo'),
                                params={"serial" : "token1"})
        print "tokeninfo response: %r" % response.testbody
        assert 'class=tokeninfoOuterTable' in response.testbody
        assert 'Heinz Hirtz' in response.testbody
        assert '<td class=tokeninfoOuterTable>\n    \t\tTokenSerialnumber\n    </td>\n    \t<!-- middle column -->\n    <td class=tokeninfoOuterTable>\n    \ttoken1\n    </td>' in response.testbody


    def test_09_logout(self):
        '''
        Manage: testing logout
        '''
        response = self.app.get(url(controller='account', action='logout'),
                                params={})
        print "logout response: %r" % response.testbody
        assert u'302 Found' in response.testbody
        
    def test_10_custom_style(self):
        '''
        Manage: testing custom style
        '''
        response = self.app.get(url(controller='manage', action='custom_style'),
                                params={})
        print "custom style: %s" % response.headers
        assert response.headers.get("Content-Type") == "text/css"



# -*- coding: utf-8 -*-
#
#    privacyIDEA is a fork of LinOTP
#    May 28, 2014 Cornelius Kölbel
#    E-mail: info@privacyidea.org
#    Contact: www.privacyidea.org
#
#    Copyright (C) LinOTP: 2010 - 2014 LSE Leading Security Experts GmbH
#
#    This program is free software: you can redistribute it and/or
#    modify it under the terms of the GNU Affero General Public
#    License, version 3, as published by the Free Software Foundation.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the
#               GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
#
#

"""
This is a SMS Provide class that can send OTP values via SMS using a phone
that is connected to the privacyIDEA server

This module makes use of the command line programm gnokii. It gets configured
in a file .gnokiirc file like this:

[global]
model = AT
port = /dev/ttyACM1
connection = serial

"""
from privacyidea.smsprovider.SMSProvider import getSMSProviderClass
from privacyidea.smsprovider.SMSProvider import ISMSProvider
import subprocess

import logging
log = logging.getLogger(__name__)


class DeviceSMSProvider(ISMSProvider):
    def __init__(self):
        self.config = {}

    def submitMessage(self, phone, message):
        '''
            submitMessage()
            - send out a message to a phone

        '''
        if (not self.config.has_key('CONFIGFILE')):
            log.error("[submitMessage] No config key CONFIGFILE found!")
            return
        else:
            configfile = self.config.get('CONFIGFILE')
            log.info("[submitMessage] setting configfile to %s" % configfile)

        # NOTE 1: The service account need rw-access to /dev/ttyXXX
        # NOTE 2: we need gnokii 0.6.29, since 0.6.28 will crash with a bug

        # FIXME: Das blockiert hier noch!
        #args = ['gnokii', "--config", configfile, "--sendsms", phone]
        #log.info("[submitMessage] preparing to run : %s" % string.join(args) )
        #proc = subprocess.Popen(args,shell=False,stdin=subprocess.PIPE)
        #log.info("process run. Now sending message as input into pipe")
        #proc.communicate(message+"\n")

        command = "echo %s | gnokii --config %s --sendsms %s" % (message, configfile, phone)
        log.debug("[submitMessage] running command: %s" % command)
        proc = subprocess.Popen([command], shell=True)

    def loadConfig(self, configDict):
        self.config = configDict
        log.info("loading config for DeviceSMSProvider")


def main(phone, message):  # pragma: no cover
    print "SMSProvider - class load test "

    # echo "text" | gnokii --config <filename> <ziel>

    config = {'CONFIGFILE':'/home/user/.gnokiirc',
              }


    sms = getSMSProviderClass("DeviceSMSProvider", "DeviceSMSProvider")()

    sms.loadConfig(config)
    ret = sms.submitMessage(phone, message)
    print sms


if __name__ == "__main__":
    phone = "+4901234567890"
    #phone      = "015154294800"
    message = "DeviceSMSProviderClass test. blocking. :-/"
    main(phone, message)
    print "... done!"




# -*- coding: utf-8 -*-
#
#    privacyIDEA
#    (c) 2014 Cornelius Kölbel
#    E-mail: info@privacyidea.org
#    Contact: www.privacyidea.org
#
#    This program is free software: you can redistribute it and/or
#    modify it under the terms of the GNU Affero General Public
#    License, version 3, as published by the Free Software Foundation.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the
#               GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
#
#
'''
The provider can be tested by running
python SipgateSMSProvider <username> <password> <phonenumber>
'''
try:
    from privacyidea.smsprovider.SMSProvider import ISMSProvider
except:
    from SMSProvider import ISMSProvider
import logging
log = logging.getLogger(__name__)
import httplib2


REQUEST_XML='''<?xml version="1.0" encoding="UTF-8"?>
<methodCall>
<methodName>samurai.SessionInitiate</methodName>
<params>
<param>
<value>
<struct>
<member>
<name>RemoteUri</name>
<value><string>sip:%s@sipgate.de</string></value>
</member>
<member>
<name>TOS</name>
<value><string>text</string></value>
</member>
<member>
<name>Content</name>
<value><string>%s</string></value>
</member>
</struct>
</value>
</param>
</params>
</methodCall>'''

class SipgateSMSProvider(ISMSProvider):

    # We not need to overwrite the __init__ and 
    # the loadConfig functions!
    # They provide the self.config dictionary.
    
    def submitMessage(self, phone, message):
        ret = False
        username = self.config.get("USERNAME")
        password = self.config.get("PASSWORD")
        http = httplib2.Http()
        http.add_credentials(username, password)
        response, content = http.request("https://samurai.sipgate.net/RPC2", "POST",
                                headers={'content-type':'text/xml'},
                                body=REQUEST_XML % (phone.strip().strip("+"), message))
        log.debug("SMS submitted: %s" % response)
        log.debug("response content: %s" % content)
        if response.get("status") == "200":
            ret = True
        else:
            log.error("Error during sending SMS: %s\n%s" % (response, content))
            ret = response.get("status")

        return ret
        

import sys
        
if __name__ == "__main__":
    username = sys.argv[1]
    password = sys.argv[2] 
    phone = sys.argv[3]
    text = "Your OTP"
    config = {'USERNAME' : username,
              'PASSWORD' : password}
    sms = SipgateSMSProvider()
    sms.loadConfig(config)
    r = sms.submitMessage(phone, text)
    print r
    

        
    
    


# -*- coding: utf-8 -*-
#  privacyIDEA is a fork of LinOTP
#  May 08, 2014 Cornelius Kölbel
#  License:  AGPLv3
#  contact:  http://www.privacyidea.org
#
#  Copyright (C) 2010 - 2014 LSE Leading Security Experts GmbH
#  License:  AGPLv3
#  contact:  http://www.linotp.org
#            http://www.lsexperts.de
#            linotp@lsexperts.de
#
# This code is free software; you can redistribute it and/or
# modify it under the terms of the GNU AFFERO GENERAL PUBLIC LICENSE
# License as published by the Free Software Foundation; either
# version 3 of the License, or any later version.
#
# This code is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU AFFERO GENERAL PUBLIC LICENSE for more details.
#
# You should have received a copy of the GNU Affero General Public
# License along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
'''
  Description:  functional tests
                
  Dependencies: -

'''
from privacyidea.tests import TestController, url


class TestYubikeyController(TestController):

    def test_yubico_mode(self):
        """
        Enroll and test the Yubikey in yubico (AES) mode
        """
        serialnum = "01382015"
        yubi_slot = 1
        serial = "UBAM%s_%s" % (serialnum, yubi_slot)
        otpkey = "9163508031b20d2fbb1868954e041729"
        parameters = {
            'type': 'yubikey',
            'serial': serial,
            'otpkey': otpkey,
            'otplen': 48,
            'description': "Yubikey enrolled in functional tests"
        }
        public_uid = "ecebeeejedecebeg"

        response = self.app.get(url(controller='admin', action='init'),
                                params=parameters)
        assert '"value": true' in response
        # test initial assign
        parameters = {"serial": serial,
                      "user": "root"}
        response = self.app.get(url(controller='admin', action='assign'),
                                params=parameters)
        # Test response...
        assert '"value": true' in response

        valid_otps = [
            public_uid + "fcniufvgvjturjgvinhebbbertjnihit",
            public_uid + "tbkfkdhnfjbjnkcbtbcckklhvgkljifu",
            public_uid + "ktvkekfgufndgbfvctgfrrkinergbtdj",
            public_uid + "jbefledlhkvjjcibvrdfcfetnjdjitrn",
            public_uid + "druecevifbfufgdegglttghghhvhjcbh",
            public_uid + "nvfnejvhkcililuvhntcrrulrfcrukll",
            public_uid + "kttkktdergcenthdredlvbkiulrkftuk",
            public_uid + "hutbgchjucnjnhlcnfijckbniegbglrt",
            public_uid + "vneienejjnedbfnjnnrfhhjudjgghckl",
            public_uid + "krgevltjnujcnuhtngjndbhbiiufbnki",
            public_uid + "kehbefcrnlfejedfdulubuldfbhdlicc",
            public_uid + "ljlhjbkejkctubnejrhuvljkvglvvlbk",
            public_uid + "eihtnehtetluntirtirrvblfkttbjuih",
        ]

        for otp in valid_otps:
            response = self.app.get(url(controller='validate',
                                        action='check_s'),
                                    params={'serial': serial, 'pass': otp})
            assert '"value": true' in response

        # Repeat an old (therefore invalid) OTP value
        invalid_otp = public_uid + "fcniufvgvjturjgvinhebbbertjnihit"
        response = self.app.get(url(controller='validate', action='check_s'),
                                params={'serial': serial, 'pass': invalid_otp})
        assert '"value": false' in response
        
        response = self.app.get(url(controller="admin", action="remove"),
                                params={'serial': serial})
        assert '"value": 1' in response

    def test_check_yubikey(self):
        '''
        Test the /validate/check_yubikey without user
        '''
        serialnum = "01382015"
        yubi_slot = 1
        serial = "UBAM%s_%s" % (serialnum, yubi_slot)
        otpkey = "9163508031b20d2fbb1868954e041729"
        parameters = {
            'type': 'yubikey',
            'serial': serial,
            'otpkey': otpkey,
            'otplen': 48,
            'description': "Yubikey enrolled in functional tests"
        }
        public_uid = "ecebeeejedecebeg"

        response = self.app.get(url(controller='admin', action='init'),
                                params=parameters)
        assert '"value": true' in response
        # test initial assign
        parameters = {"serial": serial,
                      "user": "root"}
        response = self.app.get(url(controller='admin', action='assign'),
                                params=parameters)
        # Test response...
        assert '"value": true' in response

        valid_otps = [
            public_uid + "fcniufvgvjturjgvinhebbbertjnihit",
            public_uid + "tbkfkdhnfjbjnkcbtbcckklhvgkljifu",
            public_uid + "ktvkekfgufndgbfvctgfrrkinergbtdj",
            public_uid + "jbefledlhkvjjcibvrdfcfetnjdjitrn",
            public_uid + "druecevifbfufgdegglttghghhvhjcbh",
            public_uid + "nvfnejvhkcililuvhntcrrulrfcrukll",
            public_uid + "kttkktdergcenthdredlvbkiulrkftuk",
            public_uid + "hutbgchjucnjnhlcnfijckbniegbglrt",
            public_uid + "vneienejjnedbfnjnnrfhhjudjgghckl",
            public_uid + "krgevltjnujcnuhtngjndbhbiiufbnki",
            public_uid + "kehbefcrnlfejedfdulubuldfbhdlicc",
            public_uid + "ljlhjbkejkctubnejrhuvljkvglvvlbk",
            public_uid + "eihtnehtetluntirtirrvblfkttbjuih",
        ]

        for otp in valid_otps:
            response = self.app.get(url(controller='validate',
                                        action='check_yubikey'),
                                    params={'pass': otp})
            print response
            assert '"value": true' in response
            assert '"user": "root"' in response

        response = self.app.get(url(controller="admin", action="remove"),
                                params={'serial': serial})
        print response
        assert '"value": 1' in response



# -*- coding: utf-8 -*-
#
#  privacyIDEA is a fork of LinOTP
#  May 08, 2014 Cornelius Kölbel
#  License:  AGPLv3
#  contact:  http://www.privacyidea.org
#
#  Copyright (C) 2010 - 2014 LSE Leading Security Experts GmbH
#  License:  LSE
#  contact:  http://www.linotp.org
#            http://www.lsexperts.de
#            linotp@lsexperts.de
#
# This code is free software; you can redistribute it and/or
# modify it under the terms of the GNU AFFERO GENERAL PUBLIC LICENSE
# License as published by the Free Software Foundation; either
# version 3 of the License, or any later version.
#
# This code is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU AFFERO GENERAL PUBLIC LICENSE for more details.
#
# You should have received a copy of the GNU Affero General Public
# License along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
'''
  Description:  This file contains the e-mail token implementation:
              - EmailTokenClass   (HOTP)

  Dependencies: -

'''

import logging
import traceback
import sys
import datetime

from privacyidea.lib.tokens.hmactoken import HmacTokenClass
from privacyidea.lib.config import getFromConfig
from privacyidea.lib.util import getParam
from privacyidea.lib.HMAC import HmacOtp

from privacyidea.lib.validate import split_pin_otp
from privacyidea.lib.validate import check_pin
from privacyidea.lib.validate import check_otp
from privacyidea.lib.log import log_with
from json import loads

optional = True
required = False

LOG = logging.getLogger(__name__)


class EmailTokenClass(HmacTokenClass):
    """
    E-mail token (similar to SMS token)
    """

    EMAIL_ADDRESS_KEY = "email_address"
    DEFAULT_EMAIL_PROVIDER = "privacyidea.lib.emailprovider.SMTPEmailProvider"
    DEFAULT_EMAIL_BLOCKING_TIMEOUT = 120

    def __init__(self, aToken):
        HmacTokenClass.__init__(self, aToken)
        self.setType(u"email")
        self.hKeyRequired = False

        # we support various hashlib methods, but only on create
        # which is effectively set in the update
        self.hashlibStr = getFromConfig("hotp.hashlib", "sha1")
        self.mode = ['challenge']

    @property
    def _email_address(self):
        return self.getFromTokenInfo(self.EMAIL_ADDRESS_KEY)

    @_email_address.setter
    def _email_address(self, value):
        self.addToTokenInfo(self.EMAIL_ADDRESS_KEY, value)

    @classmethod
    def getClassType(cls):
        return "email"

    @classmethod
    def getClassPrefix(cls):
        return "LSEM"

    @classmethod
    @log_with(LOG)
    def getClassInfo(cls, key=None, ret='all'):
        """
        getClassInfo - returns a subtree of the token definition

        :param key: subsection identifier
        :type key: string

        :param ret: default return value, if nothing is found
        :type ret: user defined

        :return: subsection if key exists or user defined
        :rtype: s.o.

        """
        res = {
            'type':         'email',
            'title':        'E-mail Token',
            'description':  'An e-mail token.',
            'init': {
                'page': {
                    'html': 'emailtoken.mako',
                    'scope': 'enroll',
                },
                'title': {
                    'html': 'emailtoken.mako',
                    'scope': 'enroll.title',
                },
            },
            'config': {
                'title': {
                    'html': 'emailtoken.mako',
                    'scope': 'config.title',
                },
                'page': {
                    'html': 'emailtoken.mako',
                    'scope': 'config',
                },
            },
            'selfservice': {},
            'policy': {},
        }

        # do we need to define the lost token policies here... [comment copied from sms token]
        if key is not None and key in res:
            ret = res.get(key)
        else:
            if ret == 'all':
                ret = res

        return ret

    @log_with(LOG)
    def update(self, param, reset_failcount=True):
        """
        update - process initialization parameters

        :param param: dict of initialization parameters
        :type param: dict

        :return: nothing

        """
        # specific - e-mail
        self._email_address = getParam(param, self.EMAIL_ADDRESS_KEY, optional=False)

        ## in case of the e-mail token, only the server must know the otpkey
        ## thus if none is provided, we let create one (in the TokenClass)
        if not 'genkey' in param and not 'otpkey' in param:
            param['genkey'] = 1

        HmacTokenClass.update(self, param, reset_failcount)
        return

    @log_with(LOG)
    def _getNextOtp(self):
        """
        access the nex valid otp

        :return: otpval
        :rtype: string
        """
        try:
            otplen = int(self.token.privacyIDEAOtpLen)
        except ValueError as ex:
            LOG.error("ValueError %r" % ex)
            raise Exception(ex)

        secret_obj = self.token.getHOtpKey()
        counter = self.token.getOtpCounter()

        #log.debug("serial: %s",serialNum)
        hmac2otp = HmacOtp(secret_obj, counter, otplen)
        nextotp = hmac2otp.generate(counter + 1)
        return nextotp

    @log_with(LOG)
    def initChallenge(self, transactionid, challenges=None, options=None):
        """
        initialize the challenge -
        This method checks if the creation of a new challenge (identified by transactionid)
        should proceed or if an old challenge should be used instead.

        :param transactionid: the id of the new challenge
        :param options: the request parameters

        :return: tuple of
                success - bool
                transactionid_to_use - the best transaction id for this request context
                message - which is shown to the user
                attributes - further info (dict) shown to the user
        """
        success = True
        transactionid_to_use = transactionid
        message = 'challenge init ok'
        attributes = {}

        now = datetime.datetime.now()
        blocking_time = int(getFromConfig('EmailBlockingTimeout', self.DEFAULT_EMAIL_BLOCKING_TIMEOUT))

        for challenge in challenges:
            challenge_timestamp = challenge.get('timestamp')
            assert(challenge_timestamp <= now)
            block_timeout = challenge_timestamp + datetime.timedelta(seconds=blocking_time)
            # check if there is a challenge that is blocking the creation of new challenges
            if now <= block_timeout:
                transactionid_to_use = challenge.getTransactionId()
                message = 'e-mail with otp already submitted'
                success = False
                attributes = {'info': 'challenge already submitted',
                              'state': transactionid_to_use}
                break

        return success, transactionid_to_use, message, attributes

    @log_with(LOG)
    def createChallenge(self, transactionid, options=None):
        """
        create a challenge, which is submitted to the user

        :param transactionid: the id of this challenge
        :param options: the request context parameters / data
        :return: tuple of (bool, message, data and attributes)
                 bool, if submit was successful
                 message is status-info submitted to the user
                 data is preserved in the challenge
                 attributes - additional attributes, which are displayed in the
                    output
        :rtype: bool, string, dict, dict
        """
        attributes = {}
        data = {'counter_value': "%s" % self.getOtpCount()}
        success, status_message = self._sendEmail()
        if success:
            attributes = {'state': transactionid}
        return success, status_message, data, attributes

    def _getEmailMessage(self):
        """
        Could be used to implement some more complex logic similar to the
        SMS token where the SMS text is read from a policy.

        :return: The message that is sent to the user. It should contain
            at least the placeholder <otp>
        :rtype: string
        """
        return "<otp>"

    @log_with(LOG)
    def _sendEmail(self):
        """
        Prepares the e-mail by gathering all relevant information and then sends
        it out.

        :return: A tuple of success and status_message
        :rtype: bool, string
        """
        otp = self._getNextOtp()
        email_address = self._email_address
        if not email_address:
            raise Exception("No e-mail address was defined for this token.")
        message = self._getEmailMessage()
        message = message.replace("<otp>", otp)
        message = message.replace("<serial>", self.getSerial())
        try:
            email_provider_class = self._getEmailProviderClass()
            email_provider = email_provider_class()
        except Exception as exc:
            LOG.error("Failed to load EmailProvider: %r" % exc)
            LOG.error(traceback.format_exc())
            raise exc

        ## now we need the config from the env
        LOG.debug("loading e-mail configuration for class %s" % email_provider)
        config = self._getEmailProviderConfig()
        LOG.debug("config: %r" % config)
        email_provider.loadConfig(config)
        status, status_message = email_provider.submitMessage(email_address, message)
        return status, status_message

    @log_with(LOG)
    def _getEmailProviderConfig(self):
        """
        get the defined e-mail provider config definition

        :return: dict of the e-mail provider definition
        :rtype: dict
        """
        config = {}
        tConfig = getFromConfig("encprivacyidea.EmailProviderConfig", None)
        if tConfig is None:
            tConfig = getFromConfig("EmailProviderConfig", "{}")

        if tConfig is not None:
            config = loads(tConfig)
        return config

    @log_with(LOG)
    def _getEmailProviderClass(self):
        """
        getEmailProviderClass():

        helper method to load the EmailProvider class from config

        checks, if the submitMessage method exists
        if not an error is thrown
        """
        email_provider = getFromConfig("EmailProvider", self.DEFAULT_EMAIL_PROVIDER)
        if not email_provider:
            raise Exception("No EmailProvider defined.")
        (email_provider_package, email_provider_class_name) = email_provider.rsplit(".", 1)

        if not email_provider_package or not email_provider_class_name:
            raise Exception("Could not load e-mail provider class. Maybe EmailProvider is "
                            "not set in the config file.")

        mod = __import__(email_provider_package, globals(), locals(), [email_provider_class_name])
        # TODO Kay sagt hier soll das Modul global geladen werden (mit einem bisher nicht existierenden Hook)
        provider_class = getattr(mod, email_provider_class_name)
        if not hasattr(provider_class, "submitMessage"):
            raise NameError("EmailProvider AttributeError: " + email_provider_package + "." +
                            email_provider_class_name + " instance of EmailProvider has no method 'submitMessage'")
        return provider_class

    @log_with(LOG)
    def is_challenge_response(self, passw, user, options=None, challenges=None):
        """
        Checks if the request is a challenge response.

        With the e-mail token every request has to be either a challenge
        request or a challenge response.

        Normally the client is unable to generate OTP values for this token
        himself (because the seed is generated on the server and not published)
        and has to wait to get it by e-mail. Therefore he either makes a
        challenge-request (triggering the e-mail) or he makes a challenge-
        response (sending the OTP value he received).

        :return: Is this a challenge response?
        :rtype: bool
        """
        challenge_response = False
        if options and ("state" in options or "transactionid" in options):
            challenge_response = True
        elif not self.is_challenge_request(passw, user, options):
            # If it is not a request then it is a response
            challenge_response = True

        return challenge_response

    @log_with(LOG)
    def checkResponse4Challenge(self, user, passw, options=None, challenges=None):
        """
        verify the response of a previous challenge

        There are two possible cases:

        1) The 'transaction_id' (also know as 'state', which has the same
           value) is available in options
        2) No 'transaction_id'

        In the first case we can safely assume that the passw only contains the OTP (no pin).
        In the second case passw will contain both and we split to get the OTP.

        :param user:     the requesting user
        :param passw:    the to be checked pass (pin+otp)
        :param options:  options an additional argument, which could be token
                          specific
        :param challenges: the list of challenges, where each challenge is
                            described as dict
        :return: tuple of (otpcounter and the list of matching challenges)

        """
        transaction_id = None
        otp_counter = -1
        matching_challenges = []

        if challenges is None or len(challenges) == 0:
            # There are no challenges for this token
            return -1, []

        if options and ('transactionid' in options or 'state' in options):
            ## fetch the transactionid
            transaction_id = options.get('transactionid', None)
            if transaction_id is None:
                transaction_id = options.get('state', None)

        if transaction_id:
            otp = passw
            # if the transaction_id is set we can assume that we have only received a single
            # challenge with that transaction_id thanks to
            # privacyidea.lib.validate.ValidateToken.get_challenges()
            assert(len(challenges) == 1)
            assert(transaction_id == challenges[0].getTransactionId())
        else:
            # If no transaction_id is set the request came through the WebUI and
            # we have to check all challenges
            split_status, _, otp = split_pin_otp(self, passw, user, options)
            if split_status < 0:
                raise Exception("Could not split passw")

        window = self.getOtpCountWindow()

        for challenge in challenges:
            challenge_data = challenge.getData()
            stored_counter = challenge_data.get("counter_value")
            temp_otp_counter = self.checkOtp(otp, int(stored_counter), window, options)
            if temp_otp_counter > 0:
                otp_counter = temp_otp_counter
                matching_challenges = [challenge]
                break

        # The matching_challenges list will either contain a single challenge or will be empty.
        # Returning multiple challenges is not useful in this case because all older challenges are
        # cleaned up anyway.
        return otp_counter, matching_challenges

    @log_with(LOG)
    def authenticate(self, passw, user, options=None):
        """
        The e-mail token only supports challenge response mode therefore when a 'normal
        authenticate' request arrives we return false.

        :return: pin_match, otp_counter, reply
        :rtype: bool, int, string
        """
        pin_match = False
        otp_counter = -1
        reply = None
        return pin_match, otp_counter, reply



# -*- coding: utf-8 -*-
#
#    privacyIDEA Machine Controller test suite
#
#    Copyright (C)  2014 Cornelius Kölbel, cornelius@privacyidea.org
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.

import logging
from privacyidea.tests import TestController, url
import json
import binascii

log = logging.getLogger(__name__)


class TestMachineController(TestController):

    def _create_yubikeys(self,
                         serials=["UBOM123456_1",
                                  "UBOM234567_1"],
                         machine="machine1",
                         ip="10.0.0.1",
                         otpkey="a60f076ca60e6d966f3bdcdc96f5e94c3c8efc32"):
        client = ip
        machine1 = machine
        self._create_machine(machine1,
                             ip=client)
        # create two tokens for the one machine
        for serial in serials:
            param = {'type': 'yubikey',
                     'serial': serial,
                     'otpkey': otpkey,
                     'otplen': 8,
                     'type': "TOTP"}
            response = self.app.get(url(controller='admin', action='init'),
                                    params=param)
            print response
            assert '"value": true' in response
            
    def _delete_yubikeys(self,
                         serials=["UBOM123456_1",
                                  "UBOM234567_1"],
                         machine="machine1"):
        self._delete_machine(machine)
        for serial in serials:
            self._delete_token(serial)
            
    def _create_ssh(self,
                    otpkey,
                    serials=["SSH1"],
                    machine="machine1",
                    ip="10.0.0.1"):
        client = ip
        machine1 = machine
        self._create_machine(machine1,
                             ip=client)
        # create two tokens for the one machine
        for serial in serials:
            param = {'type': 'sshkey',
                     'serial': serial,
                     'otpkey': otpkey}
            response = self.app.get(url(controller='admin', action='init'),
                                    params=param)
            print response
            assert '"value": true' in response
        
    def _delete_ssh(self,
                    serials=["SSH1"],
                    machine="machine1"):
        self._delete_machine(machine)
        for serial in serials:
            self._delete_token(serial)
        
    def _create_token(self, serial=None):
        # create a token and add this new token to the machine
        response = self.app.get(url(controller='admin', action='init'),
                                {'type': "spass",
                                 "serial": serial,
                                 "pin": "123454"})
        print response
        assert ('"status": true' in response)
        assert ('"value": true' in response)

    def _delete_token(self, serial=None):
        # create a token and add this new token to the machine
        response = self.app.get(url(controller='admin', action='remove'),
                                {"serial": serial})
        print response
        assert ('"status": true' in response)
        assert ('"value": 1' in response)

    def _create_machine(self, name, ip=None, desc=None, decommission=None,
                        admin=None):
        response = self.app.get(url(controller='machine', action='create'),
                                {'name': name,
                                 "ip": ip,
                                 "desc": desc,
                                 "decommission": decommission,
                                 "selftest_admin": admin})
        print response
        assert ('"status": true' in response)
        assert ('"value": true' in response)

    def _delete_machine(self, name, ip=None, desc=None, decommission=None):
        response = self.app.get(url(controller='machine', action='delete'),
                                {'name': name})
        print response
        assert ('"status": true' in response)
        assert ('"value": true' in response)

    def _add_token(self, name, serial, app):
        response = self.app.get(url(controller='machine', action='addtoken'),
                                {'name': name,
                                 'serial': serial,
                                 'application': app})
        print response
        assert ('"status": true' in response)
        assert ('"value": true' in response)

    def _rm_token(self, name, serial, app):
        response = self.app.get(url(controller='machine', action='deltoken'),
                                {'name': name,
                                 'serial': serial,
                                 'application': app})
        print response
        assert ('"status": true' in response)
        assert ('"value": true' in response)

    def test_filter_show(self):
        '''
        Testing the filtering of the machine listing
        '''
        machines = {"machine1": "1.2.3.4",
                    "machine2": "1.2.3.5",
                    "machine3": "1.2.3.6"}
        for m in machines.keys():
            self._create_machine(name=m, ip=machines[m])

        # list all machines
        response = self.app.get(url(controller='machine', action='show'))
        print response
        assert ("machine1" in response)
        assert ("machine2" in response)
        assert ("machine3" in response)

        response = self.app.get(url(controller='machine', action='show'),
                                {'name': "machine1"})
        print response
        assert ("machine1" in response)
        assert ("machine2" not in response)
        assert ("machine3" not in response)

        for m in machines.keys():
            self._delete_machine(name=m)

    def test_create_with_description(self):
        '''
        Create a machine with IP and description
        '''
        self._create_machine(name="newmachine",
                             ip="1.2.3.4",
                             desc="Some strange machine äää")

        response = self.app.get(url(controller='machine', action='show'),
                                {'name': "newmachine"})
        print response
        # We do not care about the database encoding here
        assert ('Some strange machine' in response)

        self._delete_machine(name="newmachine")

    def test_machine(self):
        '''Testing creating and deleting client machines'''
        name1 = "machine1"
        name2 = "machine2"
        ip1 = "2.3.4.5"

        self._create_machine(name1, ip1)

        # Creating the machine a second time should not work!
        response = self.app.get(url(controller='machine', action='create'),
                                {'name': name1,
                                 'ip': ip1})
        print response
        assert ('"status": false' in response)
        assert ('UNIQUE' in response or "IntegrityError" in response)

        response = self.app.get(url(controller='machine', action='show'))
        print response
        assert ('"name": "%s"' % name1 in response)

        # Try to delete a machine, that does not exist
        response = self.app.get(url(controller='machine', action='delete'),
                                {'name': name2})
        print response
        assert ('"status": false' in response)
        assert ('There is no machine with name=' in response)

        # delete a machine, that does exist.
        self._delete_machine(name1)

        # list the machines.
        response = self.app.get(url(controller='machine', action='show'))
        print response
        assert ('"name": "%s"' % name1 not in response)

    def test_add_token(self):
        '''
        Adding token to a machine
        '''
        name1 = "tokenmachine"
        token1 = "tok123456"

        self._create_machine(name=name1)
        # adding a token that does not exist will not work
        response = self.app.get(url(controller='machine', action='addtoken'),
                                {'name': name1,
                                 'serial': token1,
                                 'application': "ssh"})
        print response
        assert ('"status": false' in response)
        assert ('There is no token with the serial number' in response)

        self._create_token(serial=token1)

        self._add_token(name1, token1, "ssh")
        # try to add the same again
        response = self.app.get(url(controller='machine', action='addtoken'),
                                {'name': name1,
                                 'serial': token1,
                                 'application': "ssh"})
        print response
        assert ('"status": false' in response)
        assert ('UNIQUE' in response or "IntegrityError" in response)

        # add another application
        self._add_token(name1, token1, "luks")

        # show machinetoken
        response = self.app.get(url(controller='machine', action='showtoken'),
                                {})
        print "SHOWTOKEN: "
        print response
        assert ('"status": true' in response)
        assert ('"application": "ssh"' in response)
        assert ('"application": "luks"' in response)

        # delete the machines
        for app in ["ssh", "luks"]:
            self._rm_token(name1, token1, app)
        self._delete_token(token1)
        self._delete_machine(name1)

    def test_delete_token(self):
        '''
        Delete a token, that is assigned to a machine
        '''
        name1 = "machineA"
        token1 = "tokABCD"

        # create the machine
        self._create_machine(name1)
        # create the token
        self._create_token(token1)

        # add the token to the machine
        self._add_token(name1, token1, "ssh")

        # show machinetoken
        response = self.app.get(url(controller='machine', action='showtoken'),
                                {})
        print response
        assert ('"status": true' in response)
        assert ('"application": "ssh"' in response)

        # delete the token --- the machine-token binding will also be deleted!
        self._delete_token(token1)
        self._delete_machine(name1)

    def test_output(self):
        """
        Test the json output of the machine-token-config.
        """
        response = self.app.get(url(controller='machine', action='showtoken'),
                                {})
        data = json.loads(response.body)
        result = data.get("result")
        print result
        assert result.get("status") is True
        start_token_num = int(result.get("value", {}).get("total", 0))
        
        self._create_token("t1")
        self._create_token("t2")
        self._create_token("t3")
        self._create_token("t4")

        self._create_machine(name="m1", ip="1.2.3.4")
        self._create_machine(name="m2", ip="1.2.3.5")
        self._create_machine(name="m3", ip="1.2.3.6")
        self._create_machine(name="m4", ip="1.2.3.7")

        self._add_token("m1", "t1", "SSH")
        self._add_token("m1", "t2", "SSH")
        self._add_token("m1", "t3", "SSH")
        self._add_token("m2", "t1", "SSH")
        self._add_token("m3", "t2", "SSH")
        self._add_token("m4", "t3", "SSH")

        # show machinetoken
        response = self.app.get(url(controller='machine', action='showtoken'),
                                {})
        print response
        assert ('"status": true' in response)
        assert ('"application": "SSH"' in response)
        
        data = json.loads(response.body)
        result = data.get("result")
        print result
        assert result.get("status") is True
        end_token_num = int(result.get("value", {}).get("total", 0))
        assert (end_token_num - start_token_num == 6)

        self._rm_token("m1", "t1", "SSH")
        self._rm_token("m1", "t2", "SSH")
        self._rm_token("m1", "t3", "SSH")
        self._rm_token("m2", "t1", "SSH")
        self._rm_token("m3", "t2", "SSH")
        self._rm_token("m4", "t3", "SSH")

        self._delete_machine(name="m1")
        self._delete_machine(name="m2")
        self._delete_machine(name="m3")
        self._delete_machine(name="m4")

        self._delete_token("t1")
        self._delete_token("t2")
        self._delete_token("t3")
        self._delete_token("t4")

    def test_decommission(self):
        """
        Testing machine decommission date
        """
        self._create_machine("tokDec", decommission="2014-12-31")
        self._delete_machine("tokDec")

    def test_wrong_token_and_wrong_machine(self):
        """
        Testing adding non existing token to non existing machine must fail
        """
        response = self.app.get(url(controller='machine', action='addtoken'),
                                {'name': "no_machine",
                                 'serial': "no_token",
                                 'application': "luks"})
        print response
        assert ('"status": false' in response)
        assert ('There is no machine with name' in response)

    def test_with_policies(self):
        """
        Testing machine management with policies
        """
        serial = "tok1_machine"
        machine1 = "mach1"
        machine2 = "mach2"
        self._create_token(serial)
        self._create_machine(machine1, ip="10.0.0.1")

        parameters = {'name': 'ManageAll',
                      'scope': 'machine',
                      'action': 'create, delete, addtoken, gettokenapps',
                      'realm': "*",
                      'user': 'superadmin',
                      'selftest_admin': 'superadmin'}
        response = self.app.get(url(controller='system', action='setPolicy'),
                                params=parameters)
        log.error(response)
        assert '"status": true' in response

        response = self.app.get(url(controller='machine', action='create'),
                                {'name': "newmachine",
                                 'ip': '2.3.4.5',
                                 'selftest_admin': 'looser'})
        print response
        assert "You do not have the right to manage machines" in response

        self._create_machine(machine2, ip='10.0.0.2', admin="superadmin")

        response = self.app.get(url(controller='machine',
                                    action='gettokenapps'),
                                params={})
        print response
        assert '"status": true' in response

        # delete the policy
        parameters = {'name': 'ManageAll',
                      'selftest_admin': 'superadmin'}
        response = self.app.get(url(controller='system', action='delPolicy'),
                                params=parameters)
        log.error(response)
        assert '"status": true' in response

        self._delete_machine(machine1)
        self._delete_machine(machine2)
        self._delete_token(serial)

    def test_get_tokenapps(self):
        '''
        testing retrieving apps for a machine
        '''
        machine1 = "mach101"
        machine2 = "mach102"
        serial = "tok102"
        self._create_token(serial)
        self._create_machine(machine1, ip="10.0.0.1")
        self._create_machine(machine2, ip="10.0.0.2")
        self._add_token(machine1, serial, "ssh")
        self._add_token(machine1, serial, "luks")
        self._add_token(machine2, serial, "ssh")
        
        response = self.app.get(url(controller='machine',
                                    action='gettokenapps'),
                                params={"name": machine1,
                                        "client": "10.0.0.1"})
        print "=============="
        print "Get token apps"
        print "=============="
        print response
        print "=============="
        assert '"total": 2' in response
        assert '"application": "ssh",' in response
        assert '"application": "luks",' in response
        
        response = self.app.get(url(controller='machine',
                                    action='gettokenapps'),
                                params={"name": machine1,
                                        "client": "10.0.0.1",
                                        "application": "luks"})
        print response
        assert '"total": 1' in response
        assert '"application": "luks",' in response
        
        response = self.app.get(url(controller='machine',
                                    action='gettokenapps'),
                                params={"name": machine1,
                                        "client": "10.0.0.2",
                                        "application": "luks"})
        print response
        assert '"status": false' in response
        assert 'There is no machine with name' in response
        
        self._rm_token(machine2, serial, "ssh")
        self._rm_token(machine1, serial, "luks")
        self._rm_token(machine1, serial, "ssh")
        self._delete_token(serial)
        self._delete_machine(machine1)
        self._delete_machine(machine2)
        
    def test_fail_wrong_app(self):
        '''
        testing a wrong app will not assign
        '''
        machine1 = "mach1"
        serial = "tok2_machine"
        app = "does not exist"
        self._create_token(serial)
        self._create_machine(machine1)
        response = self.app.get(url(controller="machine",
                                    action="addtoken"),
                                params={"name": machine1,
                                        "application": app,
                                        "serial": serial})
        print response
        assert "Unkown application" in response
        
        self._delete_machine(machine1)
        self._delete_token(serial)
        
    def test_nist_pub_198_a_2(self):
        """
        Yubikey test vector
        http://opensource.yubico.com/yubikey-personalization/ykchalresp.1.html
        """
        machine = "toms_key"
        serial = "UBOM_toms_token"
        app = "luks"
        ip = "10.0.0.1"
        otpkey = "303132333435363738393a3b3c3d3e3f40414243"
        challenge = "Sample #2"
        result_hex = "0922d3405faa3d194f82a45830737d5cc6c75d24"
        self._create_yubikeys(serials=[serial],
                              machine=machine,
                              ip=ip,
                              otpkey=otpkey)
        self._add_token(machine, serial, app)
        print "==================== gettokenapps for yubikey ============"
        # get all auth_items for LUKS on machine without serial number
        response = self.app.get(url(controller='machine',
                                    action='gettokenapps'),
                                {"name": machine,
                                 "application": app,
                                 "serial": serial,
                                 "client": ip,
                                 "challenge": binascii.hexlify(challenge)})
        print response
        assert result_hex in response
        self._delete_token(serial)
        self._delete_machine(machine)
        
    def test_get_authentication_items(self):
        '''
        get authentication items
        '''
        serials = ["UBOM123456_1", "UBOM234567_1"]
        machine = "machine1"
        ip = "10.0.0.1"
        chal_hex = "d1835d598bc20c4ce8312ba94f046a015f7a70c48631b88f29922f1183e77873"
        resp_hex = "c7f7a081d06a738f913dce36b538091adc6d2e93"
        app = "luks"
        self._create_yubikeys(serials=serials,
                              machine=machine,
                              ip=ip)
        for serial in serials:
            self._add_token(machine, serial, app)
        
        # get all auth_items for LUKS on machine without serial number
        response = self.app.get(url(controller='machine',
                                    action='gettokenapps'),
                                params={"name": machine,
                                        "application": app,
                                        "client": ip,
                                        "challenge": chal_hex})
        print "============"
        print "gettokenapps"
        print "============"
        print response
        
        assert '"auth_item":' in response
        assert '"challenge": "%s"' % chal_hex in response
        assert '"response": "%s"' % resp_hex in response
        assert '"serial": "UBOM123456_1",' in response
        assert '"serial": "UBOM234567_1",' in response
        assert '"application": "luks",' in response
        
        self._delete_yubikeys(serials=serials)
        
    def test_add_machine_options(self):
        '''
        Adding MachineTokenOptions, slot for LUKS app...
        '''
        serials = ["UBOM123456_1", "UBOM234567_1"]
        machine = "machine1"
        ip = "10.0.0.1"
        app = "luks"
        self._create_yubikeys(serials=serials,
                              machine=machine,
                              ip=ip)

        response = self.app.get(url(controller='machine', action='addtoken'),
                                {'name': machine,
                                 'serial': serials[0],
                                 'application': app,
                                 'option_slot': 2})
        print response
        assert ('"status": true' in response)
        assert ('"value": true' in response)
        
        response = self.app.get(url(controller='machine',
                                    action='gettokenapps'),
                                params={"name": machine,
                                        "application": app,
                                        "client": ip})
        print "============"
        print "slot options"
        print "============"
        print response
        
        assert '"option_slot": "2"' in response
        
        self._delete_yubikeys(serials=serials)

    def test_add_and_remove_luks_options(self):
        """
        Add and remove luks options from a machinetoken definition
        """
        serials = ["UBOM123456_1", "UBOM234567_1"]
        machine = "machine1"
        ip = "10.0.0.1"
        app = "luks"
        self._create_yubikeys(serials=serials,
                              machine=machine,
                              ip=ip)
        # add a token
        response = self.app.get(url(controller='machine', action='addtoken'),
                                {'name': machine,
                                 'serial': serials[1],
                                 'application': app})
        assert ('"status": true' in response)
        assert ('"value": true' in response)
        # determine the machinetoken_id
        response = self.app.get(url(controller='machine',
                                    action='gettokenapps'),
                                params={"name": machine,
                                        "application": app,
                                        "client": ip})
        data = json.loads(response.body)
        print response
        assert "UBOM234567_1" in response
        
        print "=========== Add option ==============="
        mtid = data.get("result").get("value").get("machines").keys()[0]
        # add the option
        response = self.app.get(url(controller="machine", action="addoption"),
                                params={"mtid": mtid,
                                        "option_slot": 5})
        print response
        assert ('"status": true' in response)
        assert ('"value": 1' in response)
        
        print "=========== Check options ==============="
        response = self.app.get(url(controller='machine',
                                    action='gettokenapps'),
                                params={"name": machine,
                                        "application": app,
                                        "client": ip})
        print response
        assert '"option_slot": "5"' in response
        
        print "====== delete option ====================="
        response = self.app.get(url(controller="machine", action="deloption"),
                                params={"mtid": mtid,
                                        "key": "option_slot"})
        print response
        assert ('"status": true' in response)
        assert ('"value": true' in response)
        
        print "=========== Check options ==============="
        response = self.app.get(url(controller='machine',
                                    action='gettokenapps'),
                                params={"name": machine,
                                        "application": app,
                                        "client": ip})
        print response
        assert '"option_slot": "5"' not in response
        # cleanup
        self._delete_yubikeys(serials=serials)
        
    def test_delete_token_withoptions(self):
        name1 = "mach1"
        ip1 = "1.2.3.4"
        serial = "123456"
        self._create_machine(name1, ip1)
        self._create_token(serial)
        
        self._add_token(name1, serial, "ssh")
        response = self.app.get(url(controller="machine", action="addoption"),
                                params={"name": name1,
                                        "serial": serial,
                                        "application": "ssh",
                                        "option_user": "root"})
        print response
        assert('"status": true' in response)
        assert('"value": 1' in response)
        
        # Now, we can successfully delete the token!
        response = self.app.get(url(controller="machine", action="deltoken"),
                                params={"name": name1,
                                        "serial": serial,
                                        "application": "ssh"})
        print response
        assert('"status": true' in response)
        assert('"value": true' in response)
        
        self._delete_token(serial)
        self._delete_machine(name1)
        
    def test_add_and_remove_ssh_options(self):
        """
        Add and remove ssh options from a machinetoken definition
        """
        serials = ["SSH1"]
        machine = "machine1"
        ip = "10.0.0.1"
        app = "ssh"
        otpkey = ("ssh-rsa AAAAB3NzaC1yc2EAAAADAQABAAABAQDGPzCp0Xz9"
                  "TEPF0fyNts0oyymaPupsZoXnplkEaDHTxA95NF125dtj9hnTQ"
                  "2veK/ZmB8KZFY8x15ZWB/nYCjYdMa6zBXiqm/Ku1hrDJ/j7Ro"
                  "IWhIClmA2mRr5VJGcBjt5WtrG4NxG1rpc0oKVS9wlYRcC1WOY"
                  "btDN1f0hEsAsjKiMhWPLX5zUe+CwVk67K3L7Emkmd+gkn+tcU"
                  "KVVO8BZXf5wb5cTRsbiWEXSdENHtb079OnKIQc6BRMmorE1nI"
                  "4fe312Pmk0h2bVHTAU8J5T/6ukpJZyJDUYMtpi+qVtNo90Zs6"
                  "GH4fG9OKIfw0VvGD+bG9uzAJhViZJh2I8dqOeh root@privacyidea")

        self._create_ssh(otpkey)
        # assign token
        response = self.app.get(url(controller='admin', action='assign'),
                                {'serial': "SSH1",
                                 'user': 'root'})
        print response
        assert ('"value": true' in response)
        
        # add a token
        response = self.app.get(url(controller='machine', action='addtoken'),
                                {'name': machine,
                                 'serial': serials[0],
                                 'application': app})
        assert ('"status": true' in response)
        assert ('"value": true' in response)
        # determine the machinetoken_id
        response = self.app.get(url(controller='machine',
                                    action='gettokenapps'),
                                params={"name": machine,
                                        "application": app,
                                        "client": ip})
        data = json.loads(response.body)
        print response
        assert "SSH1" in response
        assert '"username": "root"' in response
        
        print "=========== Add option ==============="
        mtid = data.get("result").get("value").get("machines").keys()[0]
        # add the option
        response = self.app.get(url(controller="machine", action="addoption"),
                                params={"mtid": mtid,
                                        "option_user": "root"})
        print response
        assert ('"status": true' in response)
        assert ('"value": 1' in response)
        
        print "=========== Check options ==============="
        response = self.app.get(url(controller='machine',
                                    action='gettokenapps'),
                                params={"name": machine,
                                        "application": app,
                                        "client": ip})
        print response
        assert '"option_user": "root"' in response
        
        print "====== delete option ====================="
        response = self.app.get(url(controller="machine", action="deloption"),
                                params={"mtid": mtid,
                                        "key": "option_user"})
        print response
        assert ('"status": true' in response)
        assert ('"value": true' in response)
        
        print "=========== Check options ==============="
        response = self.app.get(url(controller='machine',
                                    action='gettokenapps'),
                                params={"name": machine,
                                        "application": app,
                                        "client": ip})
        print response
        assert '"option_user": "root"' not in response
        # cleanup
        self._delete_ssh(serials=serials)
        
    def test_ZZ(self):
        '''
        Cleanup: Delete all tokens
        '''
        self.deleteAllTokens()




# -*- coding: utf-8 -*-
#
#  privacyIDEA is a fork of LinOTP
#  May 08, 2014 Cornelius Kölbel
#  License:  AGPLv3
#  contact:  http://www.privacyidea.org
#
#  Copyright (C) 2010 - 2014 LSE Leading Security Experts GmbH
#  License:  AGPLv3
#  contact:  http://www.linotp.org
#            http://www.lsexperts.de
#            linotp@lsexperts.de
#
# This code is free software; you can redistribute it and/or
# modify it under the terms of the GNU AFFERO GENERAL PUBLIC LICENSE
# License as published by the Free Software Foundation; either
# version 3 of the License, or any later version.
#
# This code is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU AFFERO GENERAL PUBLIC LICENSE for more details.
#
# You should have received a copy of the GNU Affero General Public
# License along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
from pylons import request
from privacyidea.lib.config import getFromConfig
from privacyidea.lib.util import get_client_from_param
from privacyidea.lib.util import get_client_from_request

import logging

log = logging.getLogger(__name__)


def get_client():
    '''
    This function returns the client.

    It first tries to get the client as it is passed as the HTTP Client via REMOTE_ADDR.

    If this client Address is in a list, that is allowed to overwrite its client address (like e.g.
    a FreeRADIUS server, which will always pass the FreeRADIUS address but not the address of the
    RADIUS client) it checks for the existance of the client parameter.
    '''
    may_overwrite = []
    over_client = getFromConfig("mayOverwriteClient", "")
    log.debug("config entry mayOverwriteClient: %s" % over_client)
    try:
        may_overwrite = [ c.strip() for c in over_client.split(',') ]
    except Exception as e:
        log.warning("evaluating config entry 'mayOverwriteClient': %r" % e)

    client = get_client_from_request(request)
    log.debug("got the original client %s" % client)

    if client in may_overwrite or client == None:
        log.debug("client %s may overwrite!" % client)
        if get_client_from_param(request.params):
            client = get_client_from_param(request.params)
            log.debug("client overwritten to %s" % client)

    log.debug("returning %s" % client)
    return client


# -*- coding: utf-8 -*-
#
#  privacyIDEA
#  Jul 18, 2014 Cornelius Kölbel
#  License:  AGPLv3
#  contact:  http://www.privacyidea.org
#
# This code is free software; you can redistribute it and/or
# modify it under the terms of the GNU AFFERO GENERAL PUBLIC LICENSE
# License as published by the Free Software Foundation; either
# version 3 of the License, or any later version.
#
# This code is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU AFFERO GENERAL PUBLIC LICENSE for more details.
#
# You should have received a copy of the GNU Affero General Public
# License along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
'''
These are the applications that can be assigned to machines in privacyIDEA.

An application must be inherited from the MachineApplicationBase class.
'''
from application_base import MachineApplicationBase
from application_base import get_auth_item
from application_base import is_application_allow_bulk_call



# -*- coding:utf-8 -*-
from mako import runtime, filters, cache
UNDEFINED = runtime.UNDEFINED
__M_dict_builtin = dict
__M_locals_builtin = locals
_magic_number = 9
_modified_time = 1422216484.89673
_enable_loop = True
_template_filename = '/usr/lib/python2.7/dist-packages/privacyidea/lib/tokens/totptoken.mako'
_template_uri = '/totptoken.mako'
_source_encoding = 'utf-8'
from webhelpers.html import escape
_exports = []


def render_body(context,**pageargs):
    __M_caller = context.caller_stack._push_frame()
    try:
        __M_locals = __M_dict_builtin(pageargs=pageargs)
        c = context.get('c', UNDEFINED)
        _ = context.get('_', UNDEFINED)
        __M_writer = context.writer()
        # SOURCE LINE 2
        __M_writer(u'\n\n')
        # SOURCE LINE 4
        if c.scope == 'config.title' :
            # SOURCE LINE 5
            __M_writer(u' ')
            __M_writer(escape(_("TOTP Token Settings")))
            __M_writer(u'\n')
        # SOURCE LINE 7
        __M_writer(u'\n\n')
        # SOURCE LINE 9
        if c.scope == 'config' :
            # SOURCE LINE 10
            __M_writer(u"<script>\n\n/*\n * 'typ'_get_config_val()\n *\n * this method is called, when the token config dialog is opened\n * - it contains the mapping of config entries to the form id\n * - according to the Config entries, the form entries will be filled\n *\n */\n\n\nfunction totp_get_config_val(){\n\tvar id_map = {};\n\n    id_map['totp.timeStep']   = 'totp_timeStep';\n    id_map['totp.timeShift']  = 'totp_timeShift';\n    id_map['totp.timeWindow'] = 'totp_timeWindow';\n\n\treturn id_map;\n\n}\n\n/*\n * 'typ'_get_config_params()\n *\n * this method is called, when the token config is submitted\n * - it will return a hash of parameters for system/setConfig call\n *\n */\nfunction totp_get_config_params(){\n\n\tvar url_params ={};\n\n    url_params['totp.timeShift'] \t= $('#totp_timeShift').val();\n    url_params['totp.timeStep'] \t= $('#totp_timeStep').val();\n    url_params['totp.timeWindow'] \t= $('#totp_timeWindow').val();\n\n\treturn url_params;\n}\n\n</script>\n\n<fieldset>\n\t<legend>")
            # SOURCE LINE 54
            __M_writer(escape(_("TOTP settings")))
            __M_writer(u"</legend>\n\t<table>\n\t\t<tr><td><label for='totp_timeStep'> ")
            # SOURCE LINE 56
            __M_writer(escape(_("TOTP time Step")))
            __M_writer(u': </label></td>\n\t\t<td><input type="text" name="tot_timeStep" class="required"  id="totp_timeStep" size="2" maxlength="2"\n\t\t\ttitle=\'')
            # SOURCE LINE 58
            __M_writer(escape(_("This is the time step for time based tokens. Usually this is 30 or 60.")))
            __M_writer(u"'> sec</td></tr>\n\t\t<tr><td><label for='totp_timeShift'> ")
            # SOURCE LINE 59
            __M_writer(escape(_("TOTP time Shift")))
            __M_writer(u': </label></td>\n\t\t<td><input type="text" name="totp_timeShift" class="required"  id="totp_timeShift" size="5" maxlength="5"\n\t\t\ttitle=\'')
            # SOURCE LINE 61
            __M_writer(escape(_("This is the default time shift of the server. This should be 0.")))
            __M_writer(u"'> sec</td></tr>\n\t\t<tr><td><label for='totp_timeWindow'> ")
            # SOURCE LINE 62
            __M_writer(escape(_("TOTP time window")))
            __M_writer(u': </label></td>\n\t\t<td><input type="text" name="totp_timeWindow" class="required"  id="totp_timeWindow" size="5" maxlength="5"\n\t\t\ttitle=\'')
            # SOURCE LINE 64
            __M_writer(escape(_("This is the time privacyIDEA will calculate before and after the current time. A reasonable value is 300.")))
            __M_writer(u"'> sec</td></tr>\n\t</table>\n</fieldset>\n\n")
        # SOURCE LINE 69
        __M_writer(u'\n\n')
        # SOURCE LINE 71
        if c.scope == 'enroll.title' :
            # SOURCE LINE 72
            __M_writer(escape(_("HMAC time based")))
            __M_writer(u'\n')
        # SOURCE LINE 74
        __M_writer(u'\n')
        # SOURCE LINE 75
        if c.scope == 'enroll' :
            # SOURCE LINE 76
            __M_writer(u'<script>\n\n/*\n * \'typ\'_enroll_setup_defaults()\n *\n * this method is called when the gui becomes visible,\n * and gets the privacyidea config as a parameter, so that the\n * gui could be prepared with the server defaults\n *\n *\n */\nfunction totp_enroll_setup_defaults(config){\n\tfor (var key in config) {\n\t\tif (key == "totp.timeStep")\n\t\t{\n\t\t\t$totp_timeStep = config["totp.timeStep"];\n\t\t\t$(\'#totp_timestep\').val($totp_timeStep);\n\t\t}\n\t}\n\t$(\'#totp_key\').val(\'\');\n}\n/*\n * \'typ\'_get_enroll_params()\n *\n * this method is called, when the token  is submitted\n * - it will return a hash of parameters for admin/init call\n *\n */\nfunction totp_get_enroll_params(){\n    var params = {};\n    params[\'type\'] = \'totp\';\n   \tparams[\'description\'] = $(\'#enroll_totp_desc\').val();\n\n    if  ( $(\'#totp_key_cb\').attr(\'checked\') ) {\n\t\tparams[\'genkey\']\t= 1;\n\t\tparams[\'hashlib\']\t= \'sha1\';\n\t\tparams[\'otplen\']\t= 6;\n    } else {\n        // OTP Key\n    \tparams[\'otpkey\'] \t= $(\'#totp_key\').val();\n    }\n    params[\'timeStep\'] \t= $(\'#totp_timestep\').val();\n\n\tjQuery.extend(params, add_user_data());\n\n    return params;\n}\n</script>\n<p>')
            # SOURCE LINE 124
            __M_writer(escape(_("Please enter or copy the HMAC key.")))
            __M_writer(u'</p>\n<table><tr>\n<td><label for="totp_key" id=\'totp_key_label\'>')
            # SOURCE LINE 126
            __M_writer(escape(_("HMAC key")))
            __M_writer(u'</label></td>\n<td><input type="text" name="totp_key" id="totp_key" value="" class="text ui-widget-content ui-corner-all" /></td>\n</tr>\n<tr><td> </td><td><input type=\'checkbox\' id=\'totp_key_cb\' onclick="cb_changed(\'totp_key_cb\',[\'totp_key\',\'totp_key_label\',\'totp_key_intro\']);">\n<label for=totp_key_cb>')
            # SOURCE LINE 130
            __M_writer(escape(_("Generate HMAC key.")))
            __M_writer(u"</label></td>\n</tr>\n\n<tr>\n<td><label for='totp_timestep'>")
            # SOURCE LINE 134
            __M_writer(escape(_("timeStep")))
            __M_writer(u"</label></td>\n<td>\n\t<select id='totp_timestep'>\n\t<option value='60' >60 ")
            # SOURCE LINE 137
            __M_writer(escape(_("seconds")))
            __M_writer(u"</option>\n\t<option value='30' >30 ")
            # SOURCE LINE 138
            __M_writer(escape(_("seconds")))
            __M_writer(u'</option>\n\t</select></td>\n</tr>\n<tr>\n    <td><label for="enroll_totp_desc" id=\'enroll_totp_desc_label\'>')
            # SOURCE LINE 142
            __M_writer(escape(_("Description")))
            __M_writer(u'</label></td>\n    <td><input type="text" name="enroll_totp_desc" id="enroll_totp_desc" value="webGUI_generated" class="text" /></td>\n</tr>\n</table>\n\n')
        # SOURCE LINE 148
        __M_writer(u'\n\n')
        # SOURCE LINE 150
        if c.scope == 'selfservice.title.enroll':
            # SOURCE LINE 151
            __M_writer(escape(_("Register totp")))
            __M_writer(u'\n')
        # SOURCE LINE 153
        __M_writer(u'\n\n')
        # SOURCE LINE 155
        if c.scope == 'selfservice.enroll':
            # SOURCE LINE 156
            __M_writer(u'<script>\n\tjQuery.extend(jQuery.validator.messages, {\n\t\trequired:  "')
            # SOURCE LINE 158
            __M_writer(escape(_('required input field')))
            __M_writer(u'",\n\t\tminlength: "')
            # SOURCE LINE 159
            __M_writer(escape(_('minimum length must be greater than {0}')))
            __M_writer(u'",\n\t\tmaxlength: "')
            # SOURCE LINE 160
            __M_writer(escape(_('maximum length must be lower than {0}')))
            __M_writer(u'",\n\t\trange: \'')
            # SOURCE LINE 161
            __M_writer(escape(_("Please enter a valid init secret. It may only contain numbers and the letters A-F.")))
            __M_writer(u'\',\n\t});\n\njQuery.validator.addMethod("totp_secret", function(value, element, param){\n\tvar res1 = value.match(/^[a-fA-F0-9]+$/i);\n\tvar res2 = !value;\n    return  res1 || res2 ;\n}, \'')
            # SOURCE LINE 168
            __M_writer(escape(_("Please enter a valid init secret. It may only contain numbers and the letters A-F.")))
            __M_writer(u'\' );\n\n$(\'#form_enroll_totp\').validate({\n\tdebug: true,\n    rules: {\n        totp_secret: {\n            minlength: 40,\n            maxlength: 64,\n            number: false,\n            totp_secret: true,\n\t\t\trequired: function() {\n            \tvar res = $(\'#totp_key_cb2\').attr(\'checked\') === \'undefined\';\n            return res;\n        }\n        }\n    }\n});\n\nfunction self_totp_get_param()\n{\n\tvar urlparam = {};\n\tvar typ = \'totp\';\n\n    if  ( $(\'#totp_key_cb2\').attr(\'checked\') ) {\n    \turlparam[\'genkey\'] = 1;\n    } else {\n        // OTP Key\n        urlparam[\'otpkey\'] = $(\'#totp_secret\').val();\n    }\n\n\turlparam[\'type\'] \t= typ;\n\turlparam[\'hashlib\'] = $(\'#totp_hashlib\').val();\n\turlparam[\'otplen\'] \t= 6;\n\turlparam[\'timestep\'] \t= $(\'#totp_timestep\').val();\n\turlparam[\'description\'] = $("#totp_self_desc").val();\n\n\treturn urlparam;\n}\n\nfunction self_totp_clear()\n{\n\t$(\'#totp_secret\').val(\'\');\n\n}\nfunction self_totp_submit(){\n\n\tvar ret = false;\n\tvar params =  self_totp_get_param();\n\n\tif  ( $(\'#totp_key_cb2\').attr(\'checked\') === undefined && $(\'#form_enroll_totp\').valid() === false) {\n\t\talert(\'')
            # SOURCE LINE 218
            __M_writer(escape(_("Form data not valid.")))
            __M_writer(u'\');\n\t} else {\n\t\tenroll_token( params );\n\t\t$("#totp_key_cb2").prop("checked", false);\n\t\tcb_changed(\'totp_key_cb2\',[\'totp_secret\',\'totp_key_label2\']);\n\t\tret = true;\n\t}\n\treturn ret;\n\n}\n</script>\n<h1>')
            # SOURCE LINE 229
            __M_writer(escape(_("Enroll your TOTP Token")))
            __M_writer(u'</h1>\n<div id=\'enroll_totp_form\'>\n\t<form class="cmxform" id=\'form_enroll_totp\'>\n\t<fieldset>\n\t\t<table><tr>\n\t\t\t<td><label for=\'totp_key_cb\'>')
            # SOURCE LINE 234
            __M_writer(escape(_("Generate HMAC key")))
            __M_writer(u'</label></td>\n\t\t\t<td><input type=\'checkbox\' name=\'totp_key_cb2\' id=\'totp_key_cb2\' onclick="cb_changed(\'totp_key_cb2\',[\'totp_secret\',\'totp_key_label2\']);"></td>\n\t\t</tr><tr>\n\t\t\t<td><label id=\'totp_key_label2\' for=\'totp_secret\'>')
            # SOURCE LINE 237
            __M_writer(escape(_("Seed for HOTP token")))
            __M_writer(u'</label></td>\n\t\t\t<td><input id=\'totp_secret\' name=\'totp_secret\' class="required ui-widget-content ui-corner-all" min="40" maxlength=\'64\'/></td>\n\t\t</tr>\n')
            # SOURCE LINE 240
            if c.totp_hashlib == 1:
                # SOURCE LINE 241
                __M_writer(u"\t\t\t<input type=hidden id='totp_hashlib' value='sha1'>\n")
            # SOURCE LINE 243
            if c.totp_hashlib == 2:
                # SOURCE LINE 244
                __M_writer(u"\t\t\t<input type=hidden id='totp_hashlib' value='sha256'>\n")
            # SOURCE LINE 246
            if c.totp_hashlib == -1:
                # SOURCE LINE 247
                __M_writer(u"\t\t<tr>\n\t\t<td><label for='totp_hashlib'>")
                # SOURCE LINE 248
                __M_writer(escape(_("hashlib")))
                __M_writer(u"</label></td>\n\t\t<td><select id='totp_hashlib'>\n\t\t\t<option value='sha1' selected>sha1</option>\n\t\t\t<option value='sha256'>sha256</option>\n\t\t\t</select></td>\n\t\t</tr>\n")
            # SOURCE LINE 255
            __M_writer(u'\n')
            # SOURCE LINE 256
            if c.totp_timestep == -1:
                # SOURCE LINE 257
                __M_writer(u"\t\t\t<tr>\n\t\t\t<td><label for='totp_timestep'>")
                # SOURCE LINE 258
                __M_writer(escape(_("timeStep")))
                __M_writer(u"</label></td>\n\t\t\t<td><select id='totp_timestep'>\n\t\t\t\t<option value='30' selected>30 ")
                # SOURCE LINE 260
                __M_writer(escape(_("seconds")))
                __M_writer(u"</option>\n\t\t\t\t<option value='60'>60 ")
                # SOURCE LINE 261
                __M_writer(escape(_("seconds")))
                __M_writer(u'</option>\n\t\t\t\t</select></td>\n\t\t\t</tr>\n')
                # SOURCE LINE 264
            else:
                # SOURCE LINE 265
                __M_writer(u"\t\t\t<input type='hidden' id='totp_timestep' value='")
                __M_writer(escape(c.totp_timestep))
                __M_writer(u"'>\n")
            # SOURCE LINE 267
            __M_writer(u'\t\t<tr>\n\t\t    <td><label for="totp_self_desc" id=\'totp_self_desc_label\'>')
            # SOURCE LINE 268
            __M_writer(escape(_("Description")))
            __M_writer(u'</label></td>\n\t\t    <td><input type="text" name="totp_self_desc" id="totp_self_desc" value="self enrolled" class="text" /></td>\n\t\t</tr>\n        </table>\n\t    <button class=\'action-button\' id=\'button_enroll_totp\'\n\t    \t    onclick="self_totp_submit();">')
            # SOURCE LINE 273
            __M_writer(escape(_("enroll TOTP Token")))
            __M_writer(u'</button>\n\n    </fieldset>\n    </form>\n</div>\n\n')
        return ''
    finally:
        context.caller_stack._pop_frame()


